@@ -4,60 +4,72 @@ const domainResource = 'domains'
 const domainAliasResource = 'domainaliases'
 
 export default {
-  getDomains () {
+  getDomains() {
     return repository.get(`/${domainResource}/`)
   },
-  getDomain (domainId) {
+  getDomain(domainId) {
     return repository.get(`/${domainResource}/${domainId}/`)
   },
-  getDomainDNSDetail (domainId) {
+  getDomainDNSDetail(domainId) {
     return repository.get(`/${domainResource}/${domainId}/dns_detail/`)
   },
-  getDomainAliases (domain) {
+  getDomainDmarcAlignment(domainId, period) {
+    return repository.get(
+      `${domainResource}/${domainId}/dmarc/alignment_stats/`,
+      { params: { period } }
+    )
+  },
+  getDomainAliases(domain) {
     let url = `/${domainAliasResource}/`
     if (domain !== undefined) {
       url += `?domain=${domain}`
     }
     return repository.get(url)
   },
-  getDomainAdministrators (domainId) {
+  getDomainAdministrators(domainId) {
     return repository.get(`/${domainResource}/${domainId}/administrators/`)
   },
-  addDomainAdministrator (domainId, accountId) {
-    return repository.post(`/${domainResource}/${domainId}/administrators/add/`, { account: accountId })
+  addDomainAdministrator(domainId, accountId) {
+    return repository.post(
+      `/${domainResource}/${domainId}/administrators/add/`,
+      { account: accountId }
+    )
   },
-  removeDomainAdministrator (domainId, accountId) {
-    return repository.post(`/${domainResource}/${domainId}/administrators/remove/`, { account: accountId })
+  removeDomainAdministrator(domainId, accountId) {
+    return repository.post(
+      `/${domainResource}/${domainId}/administrators/remove/`,
+      { account: accountId }
+    )
   },
-  createDomain (data) {
+  createDomain(data) {
     return repository.post(`/${domainResource}/`, data)
   },
-  updateDomain (domainId, data) {
+  updateDomain(domainId, data) {
     return repository.put(`/${domainResource}/${domainId}/`, data)
   },
-  patchDomain (domainId, data) {
+  patchDomain(domainId, data) {
     return repository.patch(`/${domainResource}/${domainId}/`, data)
   },
-  deleteDomain (domainId, data) {
+  deleteDomain(domainId, data) {
     return repository.post(`/${domainResource}/${domainId}/delete/`, data)
   },
-  createDomainAlias (data) {
+  createDomainAlias(data) {
     return repository.post(`/${domainAliasResource}/`, data)
   },
-  updateDomainAlias (aliasId, data) {
+  updateDomainAlias(aliasId, data) {
     return repository.put(`/${domainAliasResource}/${aliasId}/`, data)
   },
-  deleteDomainAlias (aliasId) {
+  deleteDomainAlias(aliasId) {
     return repository.delete(`/${domainAliasResource}/${aliasId}/`)
   },
-  exportAll () {
+  exportAll() {
     return repository.get(`/${domainResource}/export/`)
   },
-  importFromCSV (data) {
+  importFromCSV(data) {
     return repository.post(`/${domainResource}/import/`, data, {
       headers: {
-        'Content-Type': 'multipart/form-data'
-      }
+        'Content-Type': 'multipart/form-data',
+      },
     })
-  }
+  },
 }
