@@ -2,28 +2,31 @@
 
 import ipaddress
 import os
-from typing import List
 
 from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.core import validators as dj_validators
 from django.shortcuts import get_object_or_404
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 from django.contrib.auth import password_validation
+from django.contrib.auth.models import Permission
 
 from rest_framework import serializers
 
+from modoboa.admin import signals as admin_signals
 from modoboa.admin.api.v1 import serializers as v1_serializers
 from modoboa.core import models as core_models, signals as core_signals
 from modoboa.lib import email_utils
 from modoboa.lib import exceptions as lib_exceptions
 from modoboa.lib import fields as lib_fields
 from modoboa.lib import validators, web_utils
+from modoboa.lib.permissions import get_object_owner
 from modoboa.lib.sysutils import exec_cmd
 from modoboa.parameters import tools as param_tools
 from modoboa.limits import constants as limits_constants
 from modoboa.limits import models as limits_models
+from modoboa.limits.lib import allocate_resources_from_user
 from modoboa.transport import models as transport_models
 from modoboa.transport.api.v2 import serializers as transport_serializers
 
@@ -55,7 +58,9 @@ class DomainSerializer(v1_serializers.DomainSerializer):
 
     class Meta(v1_serializers.DomainSerializer.Meta):
         fields = v1_serializers.DomainSerializer.Meta.fields + (
-            "domain_admin", "transport", "opened_alarms_count"
+            "domain_admin",
+            "transport",
+            "opened_alarms_count",
         )
 
     def __init__(self, *args, **kwargs):
@@ -66,11 +71,37 @@ class DomainSerializer(v1_serializers.DomainSerializer):
             many=True, source="domainobjectlimit_set", required=False
         )
 
+    def validate_enable_dkim(self, value):
+        """Check prerequisites."""
+        if not value:
+            return value
+        storage_dir = param_tools.get_global_parameter("dkim_keys_storage_dir")
+        if not storage_dir:
+            raise ValidationError(_("DKIM keys storage directory not configured"))
+        return value
+
     def validate(self, data):
         result = super().validate(data)
         dtype = data.get("type", "domain")
         if dtype == "relaydomain" and not data.get("transport"):
-            raise serializers.ValidationError({"transport": _("This field is required")})
+            raise serializers.ValidationError(
+                {"transport": _("This field is required")}
+            )
+        elif dtype == "domain" and data.get("transport"):
+            raise serializers.ValidationError(
+                {"transport": _("This field is valid when type is relaydomain")}
+            )
+        user = self.context["request"].user
+        if user.role == "Resellers":
+            limit = user.userobjectlimit_set.get(name="quota")
+            if limit.max_value != 0:
+                quota = data["quota"]
+                msg = _("You can't define an unlimited quota.")
+                if quota == 0:
+                    raise serializers.ValidationError({"quota": msg})
+                default_mailbox_quota = data["default_mailbox_quota"]
+                if default_mailbox_quota == 0:
+                    raise serializers.ValidationError({"default_mailbox_quota": msg})
         return result
 
     def create(self, validated_data):
@@ -84,27 +115,29 @@ class DomainSerializer(v1_serializers.DomainSerializer):
             transport.save()
             domain.transport = transport
             domain.save()
-
         if not domain_admin:
             return domain
 
+        # 0. Check user limit
+        user = self.context["request"].user
+        core_signals.can_create_object.send(
+            self.__class__, context=user, object_type="domain_admins"
+        )
         # 1. Create a domain administrator
-        username = "%s@%s" % (domain_admin["username"], domain.name)
+        username = "{}@{}".format(domain_admin["username"], domain.name)
         try:
             da = core_models.User.objects.get(username=username)
         except core_models.User.DoesNotExist:
             pass
         else:
-            raise lib_exceptions.Conflict(
-                _("User '%s' already exists") % username)
-        user = self.context["request"].user
+            raise lib_exceptions.Conflict(_("User '%s' already exists") % username)
         core_signals.can_create_object.send(
-            self.__class__, context=user, klass=models.Mailbox)
+            self.__class__, context=user, klass=models.Mailbox
+        )
         da = core_models.User(username=username, email=username, is_active=True)
         password = domain_admin.get("password")
         if password is None:
-            password = param_tools.get_global_parameter(
-                "default_password", app="core")
+            password = param_tools.get_global_parameter("default_password", app="core")
         da.set_password(password)
         da.save()
         da.role = "DomainAdmins"
@@ -114,25 +147,28 @@ class DomainSerializer(v1_serializers.DomainSerializer):
         if domain_admin["with_mailbox"]:
             dom_admin_username = domain_admin["username"]
             mb = models.Mailbox(
-                address=dom_admin_username, domain=domain,
-                user=da, use_domain_quota=True
+                address=dom_admin_username,
+                domain=domain,
+                user=da,
+                use_domain_quota=True,
             )
-            mb.set_quota(
-                override_rules=user.has_perm("admin.change_domain"))
+            mb.set_quota(override_rules=user.has_perm("admin.change_domain"))
             mb.save(creator=user)
 
             # 3. Create aliases if needed
             condition = (
-                domain.type == "domain" and
-                domain_admin["with_aliases"] and
-                dom_admin_username != "postmaster"
+                domain.type == "domain"
+                and domain_admin["with_aliases"]
+                and dom_admin_username != "postmaster"
             )
             if condition:
                 core_signals.can_create_object.send(
-                    self.__class__, context=user, klass=models.Alias)
-                address = u"postmaster@{}".format(domain.name)
+                    self.__class__, context=user, klass=models.Alias
+                )
+                address = f"postmaster@{domain.name}"
                 alias = models.Alias.objects.create(
-                    address=address, domain=domain, enabled=True)
+                    address=address, domain=domain, enabled=True
+                )
                 alias.set_recipients([mb.full_address])
                 alias.post_create(user)
 
@@ -143,7 +179,7 @@ class DomainSerializer(v1_serializers.DomainSerializer):
         """Update domain and create/update transport."""
         transport_def = validated_data.pop("transport", None)
         resources = validated_data.pop("domainobjectlimit_set", None)
-        super().update(instance, validated_data)
+        instance = super().update(instance, validated_data)
         if transport_def and instance.type == "relaydomain":
             transport = getattr(instance, "transport", None)
             created = False
@@ -161,9 +197,9 @@ class DomainSerializer(v1_serializers.DomainSerializer):
                 instance.save()
         if resources:
             for resource in resources:
-                instance.domainobjectlimit_set.filter(
-                    name=resource["name"]
-                ).update(max_value=resource["max_value"])
+                instance.domainobjectlimit_set.filter(name=resource["name"]).update(
+                    max_value=resource["max_value"]
+                )
         return instance
 
 
@@ -178,8 +214,9 @@ class AdminGlobalParametersSerializer(serializers.Serializer):
 
     # Domain settings
     enable_mx_checks = serializers.BooleanField(default=True)
-    valid_mxs = serializers.CharField(allow_blank=True)
+    valid_mxs = serializers.CharField(default="", allow_blank=True)
     domains_must_have_authorized_mx = serializers.BooleanField(default=False)
+    enable_ipv6_mx_checks = serializers.BooleanField(default=True)
     enable_spf_checks = serializers.BooleanField(default=True)
     enable_dkim_checks = serializers.BooleanField(default=True)
     enable_dmarc_checks = serializers.BooleanField(default=True)
@@ -188,16 +225,19 @@ class AdminGlobalParametersSerializer(serializers.Serializer):
     enable_dnsbl_checks = serializers.BooleanField(default=True)
     dkim_keys_storage_dir = serializers.CharField(allow_blank=True)
     dkim_default_key_length = serializers.ChoiceField(
-        default=2048, choices=constants.DKIM_KEY_LENGTHS)
+        default=2048, choices=constants.DKIM_KEY_LENGTHS
+    )
     default_domain_quota = serializers.IntegerField(default=0)
     default_domain_message_limit = serializers.IntegerField(
-        required=False, allow_null=True)
+        required=False, allow_null=True
+    )
 
     # Mailboxes settings
     handle_mailboxes = serializers.BooleanField(default=False)
     default_mailbox_quota = serializers.IntegerField(default=0)
     default_mailbox_message_limit = serializers.IntegerField(
-        required=False, allow_null=True)
+        required=False, allow_null=True
+    )
     auto_account_removal = serializers.BooleanField(default=False)
     auto_create_domain_and_mailbox = serializers.BooleanField(default=True)
     create_alias_on_mbox_rename = serializers.BooleanField(default=False)
@@ -205,31 +245,25 @@ class AdminGlobalParametersSerializer(serializers.Serializer):
     def validate_default_domain_quota(self, value):
         """Ensure quota is a positive integer."""
         if value < 0:
-            raise serializers.ValidationError(
-                _("Must be a positive integer")
-            )
+            raise serializers.ValidationError(_("Must be a positive integer"))
         return value
 
     def validate_default_mailbox_quota(self, value):
         """Ensure quota is a positive integer."""
         if value < 0:
-            raise serializers.ValidationError(
-                _("Must be a positive integer")
-            )
+            raise serializers.ValidationError(_("Must be a positive integer"))
         return value
 
     def validate_dkim_keys_storage_dir(self, value):
         """Check that directory exists."""
         if value:
-            if not os.path.isdir(value):
-                raise serializers.ValidationError(
-                    _("Directory not found.")
-                )
             code, output = exec_cmd("which openssl")
             if code:
                 raise serializers.ValidationError(
                     _("openssl not found, please make sure it is installed.")
                 )
+            if not os.path.exists(value):
+                raise serializers.ValidationError(_("Directory not found."))
         return value
 
     def validate_valid_mxs(self, value):
@@ -237,26 +271,39 @@ class AdminGlobalParametersSerializer(serializers.Serializer):
         if not value:
             return value
         try:
-            ip_addresses = [
-                ipaddress.ip_network(v.strip())
-                for v in value.split() if v.strip()
-            ]
+            [ipaddress.ip_network(v.strip()) for v in value.split() if v.strip()]
         except ValueError:
             raise serializers.ValidationError(
                 _("This field only allows valid IP addresses (or networks)")
+            ) from None
+        return value
+
+    def validate_auto_create_domain_and_mailbox(self, value):
+        """
+        Make sure the option does not get deactivated
+        while imap migration is enabled.
+        """
+        if not value and param_tools.get_global_parameter(
+            "enabled_imapmigration", "imap_migration"
+        ):
+            raise serializers.ValidationError(
+                _("Please disable imap migration before disabling this option.")
             )
         return value
 
     def validate(self, data):
         """Check MX options."""
         condition = (
-            data.get("enable_mx_checks") and
-            data.get("domains_must_have_authorized_mx") and
-            not data.get("valid_mxs"))
+            data.get("enable_mx_checks")
+            and data.get("domains_must_have_authorized_mx")
+            and not data.get("valid_mxs")
+        )
         if condition:
-            raise serializers.ValidationError({
-                "valid_mxs": _("Define at least one authorized network / address")
-            })
+            raise serializers.ValidationError(
+                {"valid_mxs": _("Define at least one authorized network / address")}
+            )
+        if not data.get("enable_mx_checks"):
+            data["enable_ipv6_mx_checks"] = False
         return data
 
 
@@ -284,6 +331,18 @@ class TagSerializer(serializers.Serializer):
     type = serializers.CharField()
 
 
+class IdPossibleActionsSerializer(serializers.Serializer):
+    """Serialiser used to get possible actions related to identities."""
+
+    name = serializers.CharField()
+    icon = serializers.CharField()
+    label = serializers.CharField()
+    type = serializers.CharField()
+    content_type = serializers.CharField(required=False)
+    url = serializers.CharField(required=False)
+    filename = serializers.CharField(required=False)
+
+
 class IdentitySerializer(serializers.Serializer):
     """Serializer used for identities."""
 
@@ -292,6 +351,25 @@ class IdentitySerializer(serializers.Serializer):
     identity = serializers.CharField()
     name_or_rcpt = serializers.CharField()
     tags = TagSerializer(many=True)
+    enabled = serializers.BooleanField()
+    possible_actions = serializers.SerializerMethodField()
+
+    def get_possible_actions(self, identity) -> list[IdPossibleActionsSerializer]:
+        if not isinstance(identity, core_models.User):
+            # Return empty action list if identity type is an alias
+            # (not used for now)
+            return []
+        actions = admin_signals.extra_account_identities_actions.send(
+            self.__class__, account=identity
+        )
+        cleaned_actions = []
+        for action in actions:
+            try:
+                serialized_data = IdPossibleActionsSerializer(action[1]).data
+                cleaned_actions.append(serialized_data)
+            except (ValidationError, AttributeError):
+                continue
+        return cleaned_actions
 
 
 class AccountResourceSerializer(serializers.ModelSerializer):
@@ -302,18 +380,34 @@ class AccountResourceSerializer(serializers.ModelSerializer):
         model = limits_models.UserObjectLimit
 
 
+class AccountMeSerializer(v1_serializers.AccountSerializer):
+    """Custom Account serializer for connected user."""
+
+    permissions = serializers.SerializerMethodField()
+
+    class Meta(v1_serializers.AccountSerializer.Meta):
+        fields = v1_serializers.AccountSerializer.Meta.fields + ("permissions",)
+
+    def get_permissions(self, user) -> list[str]:
+        """Return available permissions for given user."""
+        permissions = (
+            user.user_permissions.all() | Permission.objects.filter(group__user=user)
+        ).select_related("content_type")
+        return [f"{p.content_type.app_label}.{p.codename}" for p in permissions]
+
+
 class AccountSerializer(v1_serializers.AccountSerializer):
     """Add support for user resources."""
 
     aliases = serializers.ListField(
-        child=lib_fields.DRFEmailFieldUTF8(),
-        source="mailbox.alias_addresses"
+        child=lib_fields.DRFEmailFieldUTF8(), source="mailbox.alias_addresses"
     )
+    possible_actions = serializers.SerializerMethodField()
 
     class Meta(v1_serializers.AccountSerializer.Meta):
-        fields = (
-            v1_serializers.AccountSerializer.Meta.fields
-            + ("aliases", )
+        fields = v1_serializers.AccountSerializer.Meta.fields + (
+            "aliases",
+            "possible_actions",
         )
 
     def __init__(self, *args, **kwargs):
@@ -322,8 +416,21 @@ class AccountSerializer(v1_serializers.AccountSerializer):
             return
         self.fields["resources"] = serializers.SerializerMethodField()
 
-    def get_resources(self, account) -> List[AccountResourceSerializer]:
-        if account.role == 'SimpleUsers':
+    def get_possible_actions(self, identity) -> list[IdPossibleActionsSerializer]:
+        actions = admin_signals.extra_account_identities_actions.send(
+            self.__class__, account=identity
+        )
+        cleaned_actions = []
+        for action in actions:
+            try:
+                serialized_data = IdPossibleActionsSerializer(action[1]).data
+                cleaned_actions.append(serialized_data)
+            except (ValidationError, AttributeError):
+                continue
+        return cleaned_actions
+
+    def get_resources(self, account) -> list[AccountResourceSerializer]:
+        if account.role == "SimpleUsers":
             return []
         resources = []
         for limit in account.userobjectlimit_set.all():
@@ -343,7 +450,11 @@ class MailboxSerializer(serializers.ModelSerializer):
     class Meta:
         model = models.Mailbox
         fields = (
-            "pk", "use_domain_quota", "quota", "message_limit"
+            "pk",
+            "use_domain_quota",
+            "quota",
+            "message_limit",
+            "is_send_only",
         )
 
     def validate_quota(self, value):
@@ -355,9 +466,9 @@ class MailboxSerializer(serializers.ModelSerializer):
         method = self.context["request"].method
         if not data.get("use_domain_quota", False):
             if "quota" not in data and method != "PATCH":
-                raise serializers.ValidationError({
-                    "quota": _("This field is required")
-                })
+                raise serializers.ValidationError(
+                    {"quota": _("This field is required")}
+                )
         return data
 
 
@@ -373,8 +484,7 @@ class WritableAccountSerializer(v1_serializers.WritableAccountSerializer):
     """Add support for aliases and sender addresses."""
 
     aliases = serializers.ListField(
-        child=lib_fields.DRFEmailFieldUTF8(),
-        required=False
+        child=lib_fields.DRFEmailFieldUTF8AndEmptyUser(), required=False
     )
     mailbox = MailboxSerializer(required=False)
 
@@ -382,21 +492,32 @@ class WritableAccountSerializer(v1_serializers.WritableAccountSerializer):
         fields = tuple(
             field
             for field in v1_serializers.WritableAccountSerializer.Meta.fields
-            if field != "random_password"
+            if field not in ["random_password", "domains"]
         ) + ("aliases",)
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         if not param_tools.get_global_parameter("enable_admin_limits", app="limits"):
             return
-        self.fields["resources"] = WritableResourceSerializer(many=True, required=False)
+        del self.fields["domains"]
+        if self.context["request"].user.role in ["Resellers", "SuperAdmins"]:
+            self.fields["resources"] = WritableResourceSerializer(
+                many=True, required=False
+            )
 
     def validate_aliases(self, value):
         """Check if required domains are locals and user can access them."""
         aliases = []
+        existing_aliases = (
+            self.instance.mailbox.alias_addresses if self.instance else []
+        )
         for alias in value:
             localpart, domain = models.validate_alias_address(
-                alias, self.context["request"].user)
+                alias,
+                self.context["request"].user,
+                ignore_existing=True,
+                check_limits=alias not in existing_aliases,
+            )
             aliases.append({"localpart": localpart, "domain": domain})
         return aliases
 
@@ -405,59 +526,66 @@ class WritableAccountSerializer(v1_serializers.WritableAccountSerializer):
         master_user = data.get("master_user", False)
         role = data.get("role")
         if master_user and role != "SuperAdmins":
-            raise serializers.ValidationError({
-                "master_user": _("Not allowed for this role.")
-            })
+            raise serializers.ValidationError(
+                {"master_user": _("Not allowed for this role.")}
+            )
         if role == "SimpleUsers":
             username = data.get("username")
             if username:
                 try:
                     validators.UTF8EmailValidator()(username)
                 except ValidationError as err:
-                    raise ValidationError({"username": err.message})
+                    raise ValidationError({"username": err.message}) from None
             mailbox = data.get("mailbox")
             if mailbox is None:
                 if not self.instance:
-                    data["mailbox"] = {
-                        "use_domain_quota": True
-                    }
+                    data["mailbox"] = {"use_domain_quota": True}
         if "mailbox" in data and "username" in data:
             self.address, domain_name = email_utils.split_mailbox(data["username"])
-            self.domain = get_object_or_404(
-                models.Domain, name=domain_name)
+            self.domain = get_object_or_404(models.Domain, name=domain_name)
             creator = self.context["request"].user
             if not creator.can_access(self.domain):
                 raise serializers.ValidationError({"mailbox": _("Permission denied.")})
             if not self.instance:
                 try:
                     core_signals.can_create_object.send(
-                        sender=self.__class__, context=creator,
-                        klass=models.Mailbox)
+                        sender=self.__class__, context=creator, klass=models.Mailbox
+                    )
                     core_signals.can_create_object.send(
-                        sender=self.__class__, context=self.domain,
-                        object_type="mailboxes")
+                        sender=self.__class__,
+                        context=self.domain,
+                        object_type="mailboxes",
+                    )
                 except lib_exceptions.ModoboaException as inst:
-                    raise serializers.ValidationError({
-                        "mailbox": str(inst)
-                    })
+                    raise serializers.ValidationError({"mailbox": str(inst)}) from None
+            if len(self.address) > 64:
+                raise serializers.ValidationError(
+                    {
+                        "username": _(
+                            "The left part of an email address can't be more than 64 characters."
+                        )
+                    }
+                )
         if data.get("password") or not self.partial:
             password = data.get("password")
             if password:
                 try:
                     password_validation.validate_password(
-                        data["password"], self.instance)
+                        data["password"], self.instance
+                    )
                 except ValidationError as exc:
-                    raise serializers.ValidationError({
-                        "password": exc.messages[0]})
+                    raise serializers.ValidationError(
+                        {"password": exc.messages[0]}
+                    ) from None
             elif not self.instance:
-                raise serializers.ValidationError({
-                    "password": _("This field is required.")
-                })
+                raise serializers.ValidationError(
+                    {"password": _("This field is required.")}
+                )
         aliases = data.get("aliases")
         if aliases and "mailbox" not in data:
-            raise serializers.ValidationError({
-                "aliases": _("A mailbox is required to create aliases.")
-            })
+            raise serializers.ValidationError(
+                {"aliases": _("A mailbox is required to create aliases.")}
+            )
         domain_names = data.get("domains")
         if not domain_names:
             return data
@@ -467,9 +595,9 @@ class WritableAccountSerializer(v1_serializers.WritableAccountSerializer):
             if domain:
                 domains.append(domain)
                 continue
-            raise serializers.ValidationError({
-                "domains": _("Local domain {} does not exist").format(name)
-            })
+            raise serializers.ValidationError(
+                {"domains": _("Local domain {} does not exist").format(name)}
+            )
         data["domains"] = domains
         return data
 
@@ -493,11 +621,11 @@ class WritableAccountSerializer(v1_serializers.WritableAccountSerializer):
         self.set_permissions(user, domains)
         if aliases:
             for alias in aliases:
-                models.Alias.objects.create(
+                models.Alias.objects.modify_or_create(
+                    address="{}@{}".format(alias["localpart"], alias["domain"]),
+                    recipients=[user.username],
                     creator=creator,
                     domain=alias["domain"],
-                    address="{}@{}".format(alias["localpart"], alias["domain"]),
-                    recipients=[user.username]
                 )
         return user
 
@@ -505,7 +633,7 @@ class WritableAccountSerializer(v1_serializers.WritableAccountSerializer):
         """Update account and associated objects."""
         mailbox_data = validated_data.pop("mailbox", None)
         password = validated_data.pop("password", None)
-        domains = validated_data.pop("domains", [])
+        aliases = validated_data.pop("aliases", None)
         for key, value in validated_data.items():
             setattr(instance, key, value)
         if password:
@@ -521,14 +649,37 @@ class WritableAccountSerializer(v1_serializers.WritableAccountSerializer):
                 mailbox_data["full_address"] = validated_data["username"]
                 instance.email = validated_data["username"]
                 self._create_mailbox(creator, instance, mailbox_data)
+        if aliases is not None and "username" in validated_data:
+            # We create a list to keep track of alias we need to delete
+            alias_recipients = instance.mailbox.alias_addresses
+            for alias in aliases:
+                address = "{}@{}".format(alias["localpart"], alias["domain"])
+                models.Alias.objects.modify_or_create(
+                    address=address,
+                    recipients=[validated_data["username"]],
+                    creator=creator,
+                    domain=alias["domain"],
+                )
+                try:
+                    alias_recipients.remove(address)
+                except ValueError:
+                    continue
+            for alias_address in alias_recipients:
+                alias = models.Alias.objects.filter(
+                    address=alias_address, internal=False
+                )
+                if alias.exists():
+                    alias.first().remove_recipient_or_delete(validated_data["username"])
         instance.save()
         resources = validated_data.get("resources")
         if resources:
+            owner = get_object_owner(instance)
             for resource in resources:
-                instance.userobjectlimit_set.filter(
-                    name=resource["name"]
-                ).update(max_value=resource["max_value"])
-        self.set_permissions(instance, domains)
+                limit = instance.userobjectlimit_set.get(name=resource["name"])
+                if not owner.is_superuser:
+                    allocate_resources_from_user(limit, owner, resource["max_value"])
+                limit.max_value = resource["max_value"]
+                limit.save()
         return instance
 
 
@@ -588,3 +739,14 @@ class AlarmSerializer(serializers.ModelSerializer):
         depth = 1
         fields = "__all__"
         model = models.Alarm
+
+
+class AlarmSwitchStatusSerializer(serializers.Serializer):
+    """Serializer to switch the status of an Alarm."""
+
+    status = serializers.IntegerField()
+
+    def validate(self, data):
+        if data["status"] not in [constants.ALARM_OPENED, constants.ALARM_CLOSED]:
+            raise ValidationError(_("Unknown status"))
+        return data
