@@ -1,15 +1,13 @@
 """Admin constants."""
 
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 DNSBL_PROVIDERS = [
     "b.barracudacentral.org",
     "bl.spamcop.net",
-    "dnsbl.sorbs.net",
     "ips.backscatterer.org",
     "psbl.surriel.com",
     "sbl.spamhaus.org",
-    "spam.dnsbl.sorbs.net",
     "ubl.unsubscore.com",
     "xbl.spamhaus.org",
     "zen.spamhaus.org",
@@ -17,12 +15,7 @@ DNSBL_PROVIDERS = [
 
 # Do not run tests for these domains.
 # https://en.wikipedia.org/wiki/Top-level_domain#Reserved_domains
-RESERVED_TLD = [
-    "example",
-    "invalid",
-    "localhost",
-    "test"
-]
+RESERVED_TLD = ["example", "invalid", "localhost", "test"]
 
 DOMAIN_TYPES = [
     ("domain", _("Domain")),
@@ -34,3 +27,16 @@ DKIM_KEY_LENGTHS = [
     (2048, "2048"),
     (4096, "4096"),
 ]
+
+DKIM_WRITE_ERROR = "DKIM path error"
+DKIM_ERROR = "general DKIM generation error"
+
+ALARM_OPENED = 1
+ALARM_CLOSED = 2
+
+ALARM_STATUSES = [
+    (ALARM_OPENED, _("Opened")),
+    (ALARM_CLOSED, _("Closed")),
+]
+
+REDIS_ALARM = "redis_connection_error"
