@@ -20,20 +20,26 @@ class Alarm(models.Model):
     """A simple alarm to attach to a domain and/or mailbox."""
 
     domain = models.ForeignKey(
-        "admin.Domain", on_delete=models.CASCADE, related_name="alarms")
-    mailbox = models.ForeignKey(
-        "admin.Mailbox", on_delete=models.SET_NULL,
+        "admin.Domain",
+        on_delete=models.CASCADE,
         related_name="alarms",
-        null=True, blank=True,
+        blank=True,
+        null=True,
+    )
+    mailbox = models.ForeignKey(
+        "admin.Mailbox",
+        on_delete=models.SET_NULL,
+        related_name="alarms",
+        null=True,
+        blank=True,
     )
     created = models.DateTimeField(default=timezone.now)
     closed = models.DateTimeField(null=True, blank=True)
     status = models.IntegerField(
-        default=constants.ALARM_OPENED, choices=constants.ALARM_STATUSES,
-        db_index=True
+        default=constants.ALARM_OPENED, choices=constants.ALARM_STATUSES, db_index=True
     )
 
-    title = models.CharField(max_length=150)
+    title = models.TextField()
     internal_name = models.CharField(max_length=120)
 
     objects = models.Manager.from_queryset(AlarmQuerySet)()
@@ -42,8 +48,7 @@ class Alarm(models.Model):
         ordering = ["created"]
 
     def __str__(self):
-        return "[{}] {} - {}".format(
-            self.created, self.domain, self.get_status_display())
+        return f"[{self.created}] {self.domain} - {self.get_status_display()}"
 
     def close(self):
         """Close this alarm."""
@@ -51,5 +56,11 @@ class Alarm(models.Model):
         self.closed = timezone.now()
         self.save()
 
+    def reopen(self):
+        """Reopen this alarm."""
+        self.status = constants.ALARM_OPENED
+        self.closed = None
+        self.save()
+
 
 reversion.register(Alarm)
