@@ -6,9 +6,9 @@ from reversion import revisions as reversion
 
 from django.db import models
 from django.utils import timezone
-from django.utils.encoding import force_text, smart_text
+from django.utils.encoding import force_str, smart_str
 from django.utils.functional import cached_property
-from django.utils.translation import ugettext as _, ugettext_lazy
+from django.utils.translation import gettext as _, gettext_lazy
 
 from modoboa.core import signals as core_signals
 from modoboa.core.models import User
@@ -23,47 +23,54 @@ from . import mixins
 class Domain(mixins.MessageLimitMixin, AdminObject):
     """Mail domain."""
 
-    name = models.CharField(ugettext_lazy("name"), max_length=253, unique=True,
-                            help_text=ugettext_lazy("The domain name"))
+    name = models.CharField(
+        gettext_lazy("name"),
+        max_length=253,
+        unique=True,
+        help_text=gettext_lazy("The domain name"),
+    )
     quota = models.PositiveIntegerField(
         default=0,
     )
     default_mailbox_quota = models.PositiveIntegerField(
-        verbose_name=ugettext_lazy("Default mailbox quota"),
-        default=0
+        verbose_name=gettext_lazy("Default mailbox quota"), default=0
     )
     message_limit = models.PositiveIntegerField(
-        ugettext_lazy("Message sending limit"),
-        null=True, blank=True,
-        help_text=ugettext_lazy(
-            "Number of messages this domain can send per day"
-        )
+        gettext_lazy("Message sending limit"),
+        null=True,
+        blank=True,
+        help_text=gettext_lazy("Number of messages this domain can send per day"),
     )
     enabled = models.BooleanField(
-        ugettext_lazy("enabled"),
-        help_text=ugettext_lazy("Check to activate this domain"),
-        default=True
+        gettext_lazy("enabled"),
+        help_text=gettext_lazy("Check to activate this domain"),
+        default=True,
     )
     type = models.CharField(default="domain", max_length=20)  # NOQA:A003
     enable_dns_checks = models.BooleanField(
-        ugettext_lazy("Enable DNS checks"), default=True,
-        help_text=ugettext_lazy("Check to enable DNS checks for this domain")
+        gettext_lazy("Enable DNS checks"),
+        default=True,
+        help_text=gettext_lazy("Check to enable DNS checks for this domain"),
     )
 
     transport = models.OneToOneField(
-        "transport.Transport", null=True, on_delete=models.SET_NULL)
+        "transport.Transport", null=True, on_delete=models.SET_NULL
+    )
 
     enable_dkim = models.BooleanField(
-        ugettext_lazy("Enable DKIM signing"),
-        help_text=ugettext_lazy(
+        gettext_lazy("Enable DKIM signing"),
+        help_text=gettext_lazy(
             "If you activate this feature, a DKIM key will be "
-            "generated for this domain."),
-        default=False
+            "generated for this domain."
+        ),
+        default=False,
     )
     dkim_key_selector = models.CharField(max_length=30, default="modoboa")
     dkim_key_length = models.PositiveIntegerField(
-        ugettext_lazy("Key length"), choices=constants.DKIM_KEY_LENGTHS,
-        blank=True, null=True
+        gettext_lazy("Key length"),
+        choices=constants.DKIM_KEY_LENGTHS,
+        blank=True,
+        null=True,
     )
     dkim_public_key = models.TextField(blank=True)
     dkim_private_key_path = models.CharField(max_length=254, blank=True)
@@ -94,31 +101,13 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
     @property
     def identities_count(self) -> int:
         """Total number of identities in this domain."""
-        return (
-            self.mailbox_set.count() +
-            self.alias_set.filter(internal=False).count())
+        return self.mailbox_set.count() + self.alias_set.filter(internal=False).count()
 
     @property
     def opened_alarms_count(self) -> int:
         """Number of alarms currently opened for this domain."""
         return self.alarms.opened().count()
 
-    @property
-    def tags(self):
-        label = ""
-        for dt in constants.DOMAIN_TYPES:
-            if self.type == dt[0]:
-                label = dt[1]
-        result = [{"name": self.type, "label": label, "type": "dom"}]
-        if self.transport:
-            result.append({
-                "name": self.transport.service,
-                "label": self.transport.service,
-                "type": "srv",
-                "color": "info"
-            })
-        return result
-
     @property
     def admins(self):
         """Return the domain administrators of this domain.
@@ -128,7 +117,8 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
         return User.objects.filter(
             is_superuser=False,
             objectaccess__content_type__model="domain",
-            objectaccess__object_id=self.pk)
+            objectaccess__object_id=self.pk,
+        )
 
     @property
     def aliases(self):
@@ -165,7 +155,10 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
         errors = []
         if config["enable_mx_checks"] and not self.mxrecord_set.has_valids():
             errors.append("mx")
-        if config["enable_dnsbl_checks"] and self.dnsblresult_set.blacklisted().exists():
+        if (
+            config["enable_dnsbl_checks"]
+            and self.dnsblresult_set.blacklisted().exists()
+        ):
             errors.append("dnsbl")
         if config["enable_spf_checks"]:
             if self.spf_record is None or not self.spf_record.is_valid:
@@ -183,16 +176,6 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
             return "ok"
         return "critical"
 
-    @cached_property
-    def dnsbl_status_color(self):
-        """Shortcut to DNSBL results."""
-        if not self.dnsblresult_set.exists():
-            return "warning"
-        elif self.dnsblresult_set.blacklisted().exists():
-            return "danger"
-        else:
-            return "success"
-
     @property
     def spf_record(self):
         """Return SPF record."""
@@ -225,8 +208,7 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
             return 0
         if not self.mailbox_set.exists():
             return 0
-        return self.mailbox_set.aggregate(
-            total=models.Sum("quota"))["total"]
+        return self.mailbox_set.aggregate(total=models.Sum("quota"))["total"]
 
     @cached_property
     def allocated_quota_in_percent(self):
@@ -258,25 +240,6 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
         """Return the key used to store messages count."""
         return self.name
 
-    @property
-    def bind_format_dkim_public_key(self):
-        """Return DKIM public key using Bind format."""
-        record = "v=DKIM1;k=rsa;p=%s" % self.dkim_public_key
-        # TXT records longer than 255 characters need split into chunks
-        # split record into 74 character chunks (+2 for indent, +2 for quotes(")
-        # == 78 characters) to make more readable in text editors.
-        split_record = []
-        while record:
-            if len(record) > 74:
-                split_record.append("  \"%s\"" % record[:74])
-                record = record[74:]
-            else:
-                split_record.append("  \"%s\"" % record)
-                break
-        record = "\n".join(split_record)
-        return "{}._domainkey.{}. IN TXT (\n{})".format(
-            self.dkim_key_selector, self.name, record)
-
     def add_admin(self, account):
         """Add a new administrator to this domain.
 
@@ -284,8 +247,10 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
         """
         from modoboa.lib.permissions import grant_access_to_object
 
+        # Check if domain's limit for domain admins is not reached yet
         core_signals.can_create_object.send(
-            sender=self.__class__, context=self, object_type="domain_admins")
+            sender=self.__class__, context=self, object_type="domain_admins"
+        )
         grant_access_to_object(account, self)
         for mb in self.mailbox_set.all():
             if mb.user.has_perm("admin.add_domain"):
@@ -313,10 +278,9 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
 
     def save(self, *args, **kwargs):
         """Store current data if domain is renamed."""
-        if self.oldname != self.name:
-            self.old_mail_homes = {
-                mb.id: mb.mail_home for mb in self.mailbox_set.all()
-            }
+        # We check that the instance exists to use m2m relationship
+        if self.pk and self.oldname != self.name:
+            self.old_mail_homes = {mb.id: mb.mail_home for mb in self.mailbox_set.all()}
         if self.old_dkim_key_length != self.dkim_key_length:
             self.dkim_public_key = ""
             self.dkim_private_key_path = ""
@@ -334,12 +298,12 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
         if param_tools.get_global_parameter("auto_account_removal"):
             User.objects.filter(mailbox__domain=self).delete()
         elif self.mailbox_set.count():
-            Quota.objects.filter(username__contains="@%s" % self.name).delete()
+            Quota.objects.filter(username__contains=f"@{self.name}").delete()
             ungrant_access_to_objects(self.mailbox_set.all())
         super().delete()
 
     def __str__(self):
-        return smart_text(self.name)
+        return smart_str(self.name)
 
     def from_csv(self, user, row):
         """Create a new domain from a CSV entry.
@@ -358,38 +322,38 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
         self.name = row[1].strip().lower()
         if Domain.objects.filter(name=self.name).exists():
             raise Conflict
-        domains_must_have_authorized_mx = (
-            param_tools.get_global_parameter("domains_must_have_authorized_mx")
+        domains_must_have_authorized_mx = param_tools.get_global_parameter(
+            "domains_must_have_authorized_mx"
         )
         if domains_must_have_authorized_mx and not user.is_superuser:
             if not lib.domain_has_authorized_mx(self.name):
                 raise BadRequest(
-                    _("{}: no authorized MX record found for domain")
-                    .format(self.name)
+                    _("{}: no authorized MX record found for domain").format(self.name)
                 )
         try:
             self.quota = int(row[2].strip())
         except ValueError:
             raise BadRequest(
-                _("{}: invalid quota value for domain")
-                .format(self.name)
-            )
+                _("{}: invalid quota value for domain").format(self.name)
+            ) from None
         try:
             self.default_mailbox_quota = int(row[3].strip())
         except ValueError:
             raise BadRequest(
-                _("{}: invalid default mailbox quota value for domain")
-                .format(self.name)
-            )
+                _("{}: invalid default mailbox quota value for domain").format(
+                    self.name
+                )
+            ) from None
         if self.quota != 0 and self.default_mailbox_quota > self.quota:
             raise BadRequest(
-                _("{}: default mailbox quota cannot be greater than domain "
-                  "quota").format(self.name)
+                _(
+                    "{}: default mailbox quota cannot be greater than domain " "quota"
+                ).format(self.name)
             )
-        self.enabled = (row[4].strip().lower() in ["true", "1", "yes", "y"])
+        self.enabled = row[4].strip().lower() in ["true", "1", "yes", "y"]
         core_signals.can_create_object.send(
-            sender=self.__class__, context=user, klass=Domain,
-            instance=self)
+            sender=self.__class__, context=user, klass=Domain, instance=self
+        )
         self.save(creator=user)
 
     def to_csv_rows(self):
@@ -397,10 +361,10 @@ class Domain(mixins.MessageLimitMixin, AdminObject):
         result = [
             [
                 "domain",
-                force_text(self.name),
+                force_str(self.name),
                 self.quota,
                 self.default_mailbox_quota,
-                self.enabled
+                self.enabled,
             ]
         ]
         for dalias in self.domainalias_set.all():
