@@ -4,8 +4,8 @@ from reversion import revisions as reversion
 
 from django.contrib.contenttypes.fields import GenericRelation
 from django.db import models
-from django.utils.encoding import smart_text
-from django.utils.translation import ugettext as _, ugettext_lazy
+from django.utils.encoding import smart_str
+from django.utils.translation import gettext as _, gettext_lazy
 
 from modoboa.core import models as core_models, signals as core_signals
 from modoboa.lib.exceptions import BadRequest, Conflict
@@ -27,20 +27,24 @@ class DomainAliasManager(models.Manager):
 
 
 class DomainAlias(AdminObject):
-
     """Domain aliases."""
 
-    name = models.CharField(ugettext_lazy("name"), max_length=100, unique=True,
-                            help_text=ugettext_lazy("The alias name"))
+    name = models.CharField(
+        gettext_lazy("name"),
+        max_length=100,
+        unique=True,
+        help_text=gettext_lazy("The alias name"),
+    )
     target = models.ForeignKey(
-        Domain, verbose_name=ugettext_lazy("target"),
-        help_text=ugettext_lazy("The domain this alias points to"),
-        on_delete=models.CASCADE
+        Domain,
+        verbose_name=gettext_lazy("target"),
+        help_text=gettext_lazy("The domain this alias points to"),
+        on_delete=models.CASCADE,
     )
     enabled = models.BooleanField(
-        ugettext_lazy("enabled"),
-        help_text=ugettext_lazy("Check to activate this alias"),
-        default=True
+        gettext_lazy("enabled"),
+        help_text=gettext_lazy("Check to activate this alias"),
+        default=True,
     )
 
     owners = GenericRelation(core_models.ObjectAccess)
@@ -51,7 +55,7 @@ class DomainAlias(AdminObject):
         app_label = "admin"
 
     def __str__(self):
-        return smart_text(self.name)
+        return smart_str(self.name)
 
     def from_csv(self, user, row):
         """Create a domain alias from a CSV row
@@ -72,16 +76,16 @@ class DomainAlias(AdminObject):
         try:
             self.target = Domain.objects.get(name=domname)
         except Domain.DoesNotExist:
-            raise BadRequest(_("Unknown domain %s") % domname)
+            raise BadRequest(_("Unknown domain %s") % domname) from None
         core_signals.can_create_object.send(
-            sender="import", context=self.target, object_type="domain_aliases")
+            sender="import", context=self.target, object_type="domain_aliases"
+        )
         self.enabled = row[3].strip().lower() in ["true", "1", "yes", "y"]
         self.save(creator=user)
 
     def to_csv_row(self):
         """Export to row that can be included in a CSV file."""
-        return ["domainalias", self.name,
-                self.target.name, self.enabled]
+        return ["domainalias", self.name, self.target.name, self.enabled]
 
     def to_csv(self, csvwriter):
         """Export a domain alias using CSV format
