@@ -13,7 +13,8 @@ class MXRecordQuerySet(models.QuerySet):
 
     def has_valids(self):
         """Return managed results."""
-        if param_tools.get_global_parameter("valid_mxs").strip():
+        valid_mxs = param_tools.get_global_parameter("valid_mxs")
+        if valid_mxs and valid_mxs.strip():
             return self.filter(managed=True).exists()
         return self.exists()
 
@@ -29,8 +30,7 @@ class MXRecordManager(models.Manager):
         from .. import lib
 
         now = timezone.now()
-        records = self.get_queryset().filter(
-            domain=domain, updated__gt=now)
+        records = self.get_queryset().filter(domain=domain, updated__gt=now)
         if records.exists():
             for record in records:
                 yield record
@@ -46,8 +46,9 @@ class MXRecordManager(models.Manager):
             record = self.get_queryset().create(
                 domain=domain,
                 name="{}".format(mx_addr.strip(".")),
-                address="{}".format(mx_ip_addr),
-                updated=now + delta)
+                address=f"{mx_ip_addr}",
+                updated=now + delta,
+            )
             yield record
 
 
@@ -60,7 +61,7 @@ class MXRecord(models.Model):
     managed = models.BooleanField(default=False)
     updated = models.DateTimeField()
 
-    objects = MXRecordManager.from_queryset(MXRecordQuerySet)()
+    objects: MXRecordManager = MXRecordManager.from_queryset(MXRecordQuerySet)()
 
     def is_managed(self):
         if not param_tools.get_global_parameter("enable_mx_checks"):
@@ -68,7 +69,7 @@ class MXRecord(models.Model):
         return bool(param_tools.get_global_parameter("valid_mxs").strip())
 
     def __str__(self):
-        return "{0.name} ({0.address}) for {0.domain} ".format(self)
+        return f"{self.name} ({self.address}) for {self.domain} "
 
 
 class DNSBLQuerySet(models.QuerySet):
