@@ -2,6 +2,8 @@
 
 import socket
 
+from django.conf import settings
+
 from dns.name import Name
 from dns.rdtypes.IN.A import A
 from dns.rdtypes.ANY.MX import MX
@@ -9,13 +11,13 @@ from dns.rdtypes.ANY.TXT import TXT
 from dns.resolver import NXDOMAIN, NoAnswer, NoNameservers, Timeout
 
 
-class RRset(object):
+class RRset:
 
     def __init__(self):
         self.address = "192.0.2.1"
 
 
-class RRsetInvalid(object):
+class RRsetInvalid:
 
     def __init__(self):
         self.address = "000.0.0.0"
@@ -24,15 +26,22 @@ class RRsetInvalid(object):
 _A_RECORD = A("IN", "A", "1.2.3.4")
 _MX_RECORD_1 = MX("IN", "MX", 10, Name("mx.example.com".split(".")))
 _MX_RECORD_2 = MX("IN", "MX", 10, Name("mx2.example.com".split(".")))
+_MX_RECORD_3 = MX("IN", "MX", 10, Name("mx3.example.com".split(".")))
 _SPF_RECORD = TXT("IN", "TXT", ["v=spf1 mx -all"])
 _DMARC_RECORD = TXT("IN", "TXT", ["v=DMARC1 p=reject"])
 _DKIM_RECORD = TXT("IN", "TXT", ["v=DKIM1 p=XXXXX"])
 _BAD_MX_RECORD = MX("IN", "MX", 10, Name("bad-response.example.com".split(".")))
-_DNE_MX_RECORD = MX("IN", "MX", 10, Name(
-    "does-not-exist.example.com".split(".")))
+_DNE_MX_RECORD = MX("IN", "MX", 10, Name("does-not-exist.example.com".split(".")))
 _MX_RECORDS = [_MX_RECORD_1]
 _IP_SET_RECORDS = [RRset()]
 
+_IPV4_RECORD_REDIS = (
+    socket.AF_INET,
+    socket.SOCK_STREAM,
+    6,
+    "",
+    ("127.0.0.1", int(settings.REDIS_PORT)),
+)
 _IPV4_RECORD_1 = (socket.AF_INET, socket.SOCK_STREAM, 6, "", ("192.0.2.1", 25))
 _IPV4_RECORD_2 = (socket.AF_INET, socket.SOCK_STREAM, 6, "", ("192.0.2.2", 25))
 _IPV6_RECORD = (socket.AF_INET, socket.SOCK_STREAM, 6, "", ("2001:db8::1", 25))
@@ -40,7 +49,7 @@ _BAD_IP_RECORD = (socket.AF_INET, socket.SOCK_STREAM, 6, "", ("BAD", 25))
 _IP_RECORDS = [_IPV4_RECORD_1, _IPV6_RECORD]
 
 _POSSIBLE_DNS_RESULTS = {
-    "test3.com": [_MX_RECORD_2],
+    "test3.com": [_MX_RECORD_3],
     "no-mx.example.com": NoAnswer(),
     "does-not-exist.example.com": NXDOMAIN(),
     "timeout.example.com": Timeout(),
@@ -49,6 +58,7 @@ _POSSIBLE_DNS_RESULTS = {
     "no-lookup.example.com": [_DNE_MX_RECORD],
     "no-answer.example.com": [_DNE_MX_RECORD],
     "dns-checks.com": [_MX_RECORD_2],
+    "invalid-mx.com": [_MX_RECORD_3],
 }
 
 _POSSIBLE_DNS_RESULTS_NO_MX = {
@@ -57,13 +67,19 @@ _POSSIBLE_DNS_RESULTS_NO_MX = {
     "no-lookup.example.com": NXDOMAIN(),
     "no-answer.example.com": NoAnswer(),
     "bad-response.example.com": [RRsetInvalid()],
-    "dns-checks.com": [_SPF_RECORD, ],
+    "dns-checks.com": [
+        _SPF_RECORD,
+    ],
     "modoboa._domainkey.dns-checks.com": [_DKIM_RECORD],
-    "_dmarc.dns-checks.com": [_DMARC_RECORD, ],
+    "_dmarc.dns-checks.com": [
+        _DMARC_RECORD,
+    ],
     "autoconfig.dns-checks.com": [_A_RECORD],
     "autodiscover.dns-checks.com": [_A_RECORD],
+    "mx3.example.com": [_A_RECORD],
 }
 _POSSIBLE_IP_RESULTS = {
+    settings.REDIS_HOST: [_IPV4_RECORD_REDIS],
     "test3.com": [_IPV4_RECORD_2],
     "mx2.example.com": [_IPV4_RECORD_2],
     "bad-response.example.com": [_BAD_IP_RECORD],
@@ -73,16 +89,23 @@ _POSSIBLE_IP_RESULTS = {
     "000.0.0.0": ValueError(),
 }
 
+_POSSIBLE_DNS_IPV6_RESULTS = {
+    settings.REDIS_HOST: socket.getaddrinfo(settings.REDIS_HOST, settings.REDIS_PORT),
+    "mx3.example.com": NoAnswer(),
+}
+
 
 def mock_dns_query_result(qname, *args, **kwargs):
     if "MX" in args:
         if qname in _POSSIBLE_DNS_RESULTS:
             return get_mock_dns_query_response(_POSSIBLE_DNS_RESULTS, qname)
         return _MX_RECORDS
-    elif qname in _POSSIBLE_DNS_RESULTS_NO_MX:
+    elif "AAAA" in args:
+        if qname in _POSSIBLE_DNS_IPV6_RESULTS:
+            return get_mock_dns_query_response(_POSSIBLE_DNS_IPV6_RESULTS, qname)
+    if qname in _POSSIBLE_DNS_RESULTS_NO_MX:
         return get_mock_dns_query_response(_POSSIBLE_DNS_RESULTS_NO_MX, qname)
-    else:
-        return _IP_SET_RECORDS
+    return _IP_SET_RECORDS
 
 
 def get_mock_dns_query_response(responses, attributeName):
