@@ -1,18 +1,29 @@
 """Core API related tests."""
 
 import copy
+import getpass
 import oath
 from unittest import mock
 
+from django.core.cache import cache
+from django.test import override_settings
 from django.urls import reverse
 from django.utils.encoding import force_bytes
 from django.utils.http import urlsafe_base64_encode
 
-from modoboa.admin import factories
-from modoboa.core import models
+from modoboa.admin import (
+    factories,
+    models as admin_models,
+    constants as admin_constants,
+)
+from modoboa.core import models, constants
+from modoboa.core.tests import utils
 from modoboa.lib.tests import ModoAPITestCase
 from rest_framework.authtoken.models import Token
 
+DOVEADM_TEST_PATH = utils.get_doveadm_test_path()
+DOVECOT_USER = getpass.getuser()
+
 CORE_SETTINGS = {
     "authentication_type": "local",
     "password_scheme": "sha512crypt",
@@ -59,22 +70,31 @@ CORE_SETTINGS = {
     "send_new_versions_email": False,
     "new_versions_email_rcpt": "postmaster@yourdomain.test",
     "send_statistics": True,
+    "enable_inactive_accounts": True,
     "inactive_account_threshold": 30,
     "top_notifications_check_interval": 30,
     "log_maximum_age": 365,
     "items_per_page": 30,
-    "default_top_redirection": "user"
+    "default_top_redirection": "user",
 }
 
 
 class ParametersAPITestCase(ModoAPITestCase):
+    def setUp(self):
+        super().setUp()
+        cache.delete("password_scheme_choice")
 
+    @override_settings(
+        DOVEADM_LOOKUP_PATH=[DOVEADM_TEST_PATH], DOVECOT_USER=DOVECOT_USER
+    )
     def test_update(self):
-        url = reverse("v2:parameter-detail", args=["core"])
+        url = reverse("v2:parameter-global-detail", args=["core"])
         data = copy.copy(CORE_SETTINGS)
         resp = self.client.put(url, data, format="json")
         self.assertEqual(resp.status_code, 200)
 
+        self.assertIsNotNone(cache.get("password_scheme_choice"))
+        self.assertIn(("plain", "plain (weak)"), cache.get("password_scheme_choice"))
         # Modify SMS related settings
         data["sms_password_recovery"] = True
         resp = self.client.put(url, data, format="json")
@@ -83,20 +103,19 @@ class ParametersAPITestCase(ModoAPITestCase):
         data["sms_provider"] = "test"
         resp = self.client.put(url, data, format="json")
         self.assertEqual(resp.status_code, 400)
-        data.update({
-            "sms_provider": "ovh",
-            "sms_ovh_application_key": "key",
-            "sms_ovh_application_secret": "secret",
-            "sms_ovh_consumer_key": "consumer"
-        })
+        data.update(
+            {
+                "sms_provider": "ovh",
+                "sms_ovh_application_key": "key",
+                "sms_ovh_application_secret": "secret",
+                "sms_ovh_consumer_key": "consumer",
+            }
+        )
         resp = self.client.put(url, data, format="json")
         self.assertEqual(resp.status_code, 200)
 
         # Modify some LDAP settings
-        data.update({
-            "authentication_type": "ldap",
-            "ldap_auth_method": "searchbind"
-        })
+        data.update({"authentication_type": "ldap", "ldap_auth_method": "searchbind"})
         resp = self.client.put(url, data, format="json")
         self.assertEqual(resp.status_code, 400)
         self.assertIn("ldap_search_base", resp.json())
@@ -106,17 +125,64 @@ class ParametersAPITestCase(ModoAPITestCase):
         self.assertEqual(resp.status_code, 400)
         self.assertIn("ldap_user_dn_template", resp.json())
 
-        data.update({
-            "ldap_user_dn_template": "%(user)s",
-            "ldap_sync_account_dn_template": "%(user)s",
-            "ldap_search_filter": "mail=%(user)s"
-        })
+        data.update(
+            {
+                "ldap_user_dn_template": "%(user)s",
+                "ldap_sync_account_dn_template": "%(user)s",
+                "ldap_search_filter": "mail=%(user)s",
+            }
+        )
         resp = self.client.put(url, data, format="json")
         self.assertEqual(resp.status_code, 200)
 
+    def test_doveadm_alarm(self):
+        """Test that an alarm is opened, closed or reopened
+        depending on the result of the doveadm command for the password scheme
+        """
+        # Test case where doveadm command fails
+        url = reverse("v2:parameter-global-detail", args=["core"])
+        data = copy.copy(CORE_SETTINGS)
+        data["password_scheme"] = "plain"
+
+        resp = self.client.put(url, data, format="json")
+        self.assertEqual(resp.status_code, 200)
+
+        self.assertIsNone(cache.get("password_scheme_choice"))
+        doveadm_alarm = admin_models.Alarm.objects.filter(
+            internal_name=constants.DOVEADM_PASS_SCHEME_ALARM
+        )
+        self.assertEqual(doveadm_alarm.count(), 1)
+        cache.delete("job_cache_available_password_hasher")
+
+        with self.settings(
+            DOVEADM_LOOKUP_PATH=[DOVEADM_TEST_PATH], DOVECOT_USER=DOVECOT_USER
+        ):
+            # The command should work and close the alarm
+            resp = self.client.put(url, data, format="json")
+            self.assertEqual(resp.status_code, 200)
+
+            self.assertIsNotNone(cache.get("password_scheme_choice"))
+            doveadm_alarm = admin_models.Alarm.objects.filter(
+                internal_name=constants.DOVEADM_PASS_SCHEME_ALARM
+            )
+            self.assertEqual(doveadm_alarm.count(), 1)
+            self.assertEqual(doveadm_alarm.first().status, admin_constants.ALARM_CLOSED)
+            cache.delete("job_cache_available_password_hasher")
+
+        # And lastly check that the alarm is reopened if the issue starts again
+        # Simulate that the cache has expired
+        cache.delete("password_scheme_choice")
+        resp = self.client.put(url, data, format="json")
+        self.assertEqual(resp.status_code, 200)
+
+        doveadm_alarm = admin_models.Alarm.objects.filter(
+            internal_name=constants.DOVEADM_PASS_SCHEME_ALARM
+        )
+        self.assertEqual(doveadm_alarm.count(), 1)
+        self.assertEqual(doveadm_alarm.first().status, admin_constants.ALARM_OPENED)
+
 
 class AccountViewSetTestCase(ModoAPITestCase):
-
     def test_me(self):
         url = reverse("v2:account-me")
         resp = self.client.get(url)
@@ -131,22 +197,9 @@ class AccountViewSetTestCase(ModoAPITestCase):
         resp = self.client.post(url, {"password": password_ok}, format="json")
         self.assertEqual(resp.status_code, 200)
 
-    @mock.patch("django_otp.match_token")
-    def test_tfa_verify_code(self, match_mock):
-        user = models.User.objects.get(username="admin")
-        user.totpdevice_set.create(name="Device")
-        user.tfa_enabled = True
-        user.save()
-
-        url = reverse("v2:account-tfa-verify-code")
-        match_mock.side_effect = [user.totpdevice_set.first()]
-        data = {"code": "1234"}
-        resp = self.client.post(url, data, format="json")
-        self.assertEqual(resp.status_code, 200)
-        self.assertIn("access", resp.json())
-
-    def test_tfa_setup_get_qr_code(self):
-        url = reverse("v2:account-tfa-setup-get-qr-code")
+    def test_tfa_setup(self):
+        # Setup TFA
+        url = reverse("v2:account-tfa-setup-get-key")
 
         resp = self.client.get(url)
         self.assertEqual(resp.status_code, 404)
@@ -155,15 +208,17 @@ class AccountViewSetTestCase(ModoAPITestCase):
         user.totpdevice_set.create(name="Device")
         resp = self.client.get(url)
         self.assertEqual(resp.status_code, 200)
-        self.assertEqual(resp.content_type, "application/xml")
+        self.assertEqual(resp.content_type, "application/json")
 
-        user.tfa_enabled = True
+        user.totp_enabled = True
         user.save()
         resp = self.client.get(url)
         self.assertEqual(resp.status_code, 404)
 
     @mock.patch("django_otp.plugins.otp_totp.models.TOTPDevice.verify_token")
-    def test_tfa_setup_check(self, verify_mock):
+    def test_tfa_setup_modify(
+        self, verify_mock, password_ko="Toto1234", password_ok="password"
+    ):
         user = models.User.objects.get(username="admin")
         user.totpdevice_set.create(name="Device")
 
@@ -182,27 +237,29 @@ class AccountViewSetTestCase(ModoAPITestCase):
         resp = self.client.post(url, data, format="json")
         self.assertEqual(resp.status_code, 400)
 
-    def test_api_token(self):
-        # 1. Obtain a JWT token so we can safely play with basic token
-        url = reverse("v2:token_obtain_pair")
-        data = {
-            "username": "admin",
-            "password": "password"
-        }
-        resp = self.client.post(url, data, format="json")
-        self.assertEqual(resp.status_code, 200)
-        self.client.credentials(
-            HTTP_AUTHORIZATION="Bearer {}".format(resp.json()["access"])
-        )
+        url_reset = reverse("v2:account-tfa-reset-codes")
+        url_disable = reverse("v2:account-tfa-disable")
+        data = {"password": password_ko}
+        # Try regenerate TFA backup code with wrong password
+        resp = self.client.post(url_reset, data, format="json")
+        self.assertEqual(resp.status_code, 400)
+        # Try disable with wrong password
+        resp = self.client.post(url_disable, data, format="json")
+        self.assertEqual(resp.status_code, 400)
 
-        url = reverse("v2:account-manage-api-token")
-        resp = self.client.get(url)
+        # Test with password ok
+        data = {"password": password_ok}
+        # Try regenerate TFA backup codes with good password
+        resp = self.client.post(url_reset, data, format="json")
         self.assertEqual(resp.status_code, 200)
-        self.assertEqual(resp.json()["token"], self.token.key)
-        resp = self.client.delete(url)
-        self.assertEqual(resp.status_code, 204)
-        resp = self.client.post(url)
-        self.assertEqual(resp.status_code, 201)
+        self.assertIn("tokens", resp.json())
+
+        # Try disable with good password
+        resp = self.client.post(url_disable, data, format="json")
+        self.assertEqual(resp.status_code, 200)
+
+        user.refresh_from_db()
+        self.assertEqual(user.totp_enabled, False)
 
 
 class PasswordResetTestCase(AccountViewSetTestCase):
@@ -217,7 +274,8 @@ class PasswordResetTestCase(AccountViewSetTestCase):
         super().setUpTestData()
         factories.populate_database()
         cls.da_token = Token.objects.create(
-            user=models.User.objects.get(username="admin@test.com"))
+            user=models.User.objects.get(username="admin@test.com")
+        )
 
     @mock.patch("ovh.Client.get")
     @mock.patch("ovh.Client.post")
@@ -250,13 +308,16 @@ class PasswordResetTestCase(AccountViewSetTestCase):
         self.assertEqual(response.json()["type"], "email")
 
         # SMS reset test
-        self.set_global_parameters({
-            "sms_password_recovery": True,
-            "sms_provider": "ovh",
-            "sms_ovh_application_key": "key",
-            "sms_ovh_application_secret": "secret",
-            "sms_ovh_consumer_key": "consumer"
-        }, app="core")
+        self.set_global_parameters(
+            {
+                "sms_password_recovery": True,
+                "sms_provider": "ovh",
+                "sms_ovh_application_key": "key",
+                "sms_ovh_application_secret": "secret",
+                "sms_ovh_consumer_key": "consumer",
+            },
+            app="core",
+        )
 
         # Phone number
         account.phone_number = "+33612345678"
@@ -284,13 +345,15 @@ class PasswordResetTestCase(AccountViewSetTestCase):
         # Send sms
         client_get.return_value = ["service"]
         client_post.return_value = {"totalCreditsRemoved": 1}
-        self.set_global_parameters({
-            "sms_password_recovery": True,
-            "sms_provider": "ovh",
-            "sms_ovh_application_key": "key",
-            "sms_ovh_application_secret": "secret",
-            "sms_ovh_consumer_key": "consumer"
-        })
+        self.set_global_parameters(
+            {
+                "sms_password_recovery": True,
+                "sms_provider": "ovh",
+                "sms_ovh_application_key": "key",
+                "sms_ovh_application_secret": "secret",
+                "sms_ovh_consumer_key": "consumer",
+            }
+        )
         self.client.logout()
         self.create_session()
         data = {"email": account.email}
@@ -336,13 +399,15 @@ class PasswordResetTestCase(AccountViewSetTestCase):
 
         client_get.return_value = ["service"]
         client_post.return_value = {"totalCreditsRemoved": 1}
-        self.set_global_parameters({
-            "sms_password_recovery": True,
-            "sms_provider": "ovh",
-            "sms_ovh_application_key": "key",
-            "sms_ovh_application_secret": "secret",
-            "sms_ovh_consumer_key": "consumer"
-        })
+        self.set_global_parameters(
+            {
+                "sms_password_recovery": True,
+                "sms_provider": "ovh",
+                "sms_ovh_application_key": "key",
+                "sms_ovh_application_secret": "secret",
+                "sms_ovh_consumer_key": "consumer",
+            }
+        )
 
         # Prepare account
         account = models.User.objects.get(username="user@test.com")
@@ -366,77 +431,60 @@ class PasswordResetTestCase(AccountViewSetTestCase):
         token_ok = response.json()["token"]
 
         # Password differs
-        data = {"new_password1": "123456", "new_password2": "123457", "token": token_ok, "id": id_ok}
+        data = {
+            "new_password1": "123456",
+            "new_password2": "123457",
+            "token": token_ok,
+            "id": id_ok,
+        }
         response = self.client.post(url, data, format="json")
         self.assertEqual(response.status_code, 400)
 
         # Wrong ID
         id_ko = urlsafe_base64_encode(force_bytes(-1))
-        data = {"new_password1": "123456", "new_password2": "123456", "token": token_ok, "id": id_ko}
+        data = {
+            "new_password1": "123456",
+            "new_password2": "123456",
+            "token": token_ok,
+            "id": id_ko,
+        }
         response = self.client.post(url, data, format="json")
         self.assertEqual(response.status_code, 404)
 
         # Wrong Token
         token_ko = "123456"
-        data = {"new_password1": "123456", "new_password2": "123456", "token": token_ko, "id": id_ok}
+        data = {
+            "new_password1": "123456",
+            "new_password2": "123456",
+            "token": token_ko,
+            "id": id_ok,
+        }
         response = self.client.post(url, data, format="json")
         self.assertEqual(response.status_code, 403)
 
         # Failed password requirements
-        data = {"new_password1": "123456", "new_password2": "123456", "token": token_ok, "id": id_ok}
+        data = {
+            "new_password1": "123456",
+            "new_password2": "123456",
+            "token": token_ok,
+            "id": id_ok,
+        }
         response = self.client.post(url, data, format="json")
         self.assertEqual(response.json()["type"], "password_requirement")
 
         # All good
-        data = {"new_password1": "MyHardenedPass1!", "new_password2": "MyHardenedPass1!", "token": token_ok,
-                "id": id_ok}
+        data = {
+            "new_password1": "MyHardenedPass1!",
+            "new_password2": "MyHardenedPass1!",
+            "token": token_ok,
+            "id": id_ok,
+        }
         response = self.client.post(url, data, format="json")
         self.assertEqual(response.status_code, 200)
         # TODO: See why user doesn't update it's password --> self.test_me_password(password_ok="MyHardenedPass1!")
 
 
-
-class AuthenticationTestCase(ModoAPITestCase):
-
-    @mock.patch("django_otp.match_token")
-    def test_2fa(self, match_mock):
-        url = reverse("v2:token_obtain_pair")
-        me_url = reverse("v2:account-me")
-        data = {
-            "username": "admin",
-            "password": "password"
-        }
-        resp = self.client.post(url, data, format="json")
-        self.assertEqual(resp.status_code, 200)
-
-        self.client.credentials(
-            HTTP_AUTHORIZATION="Bearer {}".format(resp.json()["access"]))
-        resp = self.client.get(me_url)
-        self.assertEqual(resp.status_code, 200)
-
-        # Now we enable 2FA
-        user = models.User.objects.get(username="admin")
-        user.totpdevice_set.create(name="Device")
-        user.tfa_enabled = True
-        user.save()
-        resp = self.client.get(me_url)
-        self.assertEqual(resp.status_code, 418)
-
-        # Verify code
-        url = reverse("v2:account-tfa-verify-code")
-        match_mock.side_effect = [user.totpdevice_set.first()]
-        data = {"code": "1234"}
-        resp = self.client.post(url, data, format="json")
-        self.assertEqual(resp.status_code, 200)
-
-        self.client.credentials(
-            HTTP_AUTHORIZATION="Bearer {}".format(resp.json()["access"]))
-        resp = self.client.get(me_url)
-        self.assertEqual(resp.status_code, 200)
-
-
 class LanguageViewSetTestCase(ModoAPITestCase):
-
     def test_list(self):
         url = reverse("v2:language-list")
         resp = self.client.get(url)
@@ -444,8 +492,121 @@ class LanguageViewSetTestCase(ModoAPITestCase):
 
 
 class ComponentAPITestCase(ModoAPITestCase):
-
     def test_information(self):
         url = reverse("v2:components_information")
         resp = self.client.get(url)
         self.assertEqual(resp.status_code, 200)
+
+
+class NotificationAPITestCase(ModoAPITestCase):
+
+    def test_get_notifications(self):
+        url = reverse("v2:notifications")
+        resp = self.client.get(url)
+        self.assertEqual(resp.status_code, 200)
+        data = resp.json()
+        self.assertEqual(len(data), 0)
+
+
+class AuthenticatorData(bytes):
+
+    credential_data = b"RESPONSE"
+
+
+class FIDOViewSetTestCase(ModoAPITestCase):
+
+    @mock.patch("fido2.server.Fido2Server.register_complete")
+    def test_registration(self, register_complete_mock):
+        url = reverse("v2:fido-registration-begin")
+        resp = self.client.post(url)
+        self.assertEqual(resp.status_code, 200)
+
+        register_complete_mock.side_effect = [AuthenticatorData()]
+        data = {
+            "type": "type",
+            "id": "XX",
+            "rawId": "XX",
+            "authenticatorAttachment": "attachment",
+            "response": {"key": "value"},
+            "name": "Name",
+        }
+        url = reverse("v2:fido-registration-end")
+        resp = self.client.post(url, data, format="json")
+        self.assertEqual(resp.status_code, 200)
+        self.assertIn("tokens", resp.json())
+
+
+class CapabilitiesAPITestCase(ModoAPITestCase):
+
+    def test_get(self):
+        url = reverse("v2:capabilities")
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertIn("rspamd", response.json()["capabilities"])
+
+
+class ThemeAPITestCase(ModoAPITestCase):
+
+    def test_get(self):
+        url = reverse("v2:theme")
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        theme = response.json()
+        self.assertEqual(theme["theme_primary_color"], "#046BF8")
+
+        self.set_global_parameter("theme_primary_color", "#FF0000")
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        theme = response.json()
+        self.assertEqual(theme["theme_primary_color"], "#FF0000")
+
+
+class NewsFeedAPIViewTestCase(ModoAPITestCase):
+
+    @classmethod
+    def setUpTestData(cls):
+        """Create test data."""
+        super().setUpTestData()
+        factories.populate_database()
+
+    def test_get(self):
+        url = reverse("v2:news-feed")
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(len(response.json()), 5)
+
+    @override_settings(DISABLE_DASHBOARD_EXTERNAL_QUERIES=True)
+    def test_if_disabling_from_settings_works(self):
+        url = reverse("v2:news-feed")
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(len(response.json()), 0)
+
+    def test_custom_rss_feed(self):
+        self.set_global_parameter(
+            "rss_feed_url", "https://www.djangoproject.com/rss/weblog/"
+        )
+        # Try as superadmin
+        url = reverse("v2:news-feed")
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertIn("modoboa", response.json()[0]["link"])
+
+        # Try as superadmin and show_rss_feed_to_superadmins enabled
+        self.set_global_parameter("show_rss_feed_to_superadmins", True)
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertIn("django", response.json()[0]["link"])
+
+        # Try as domainadmin
+        dadmin = models.User.objects.get(username="admin@test.com")
+        self.authenticate_user(dadmin)
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertIn("django", response.json()[0]["link"])
+
+        # Try fallback
+        self.set_global_parameter("rss_feed_url", "")
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertIn("modoboa", response.json()[0]["link"])
