@@ -2,7 +2,7 @@
 
 from django.apps import AppConfig
 from django.db.models import signals
-from django.utils.translation import ugettext_lazy
+from django.utils.translation import gettext_lazy
 
 
 def load_core_settings():
@@ -16,17 +16,17 @@ def load_core_settings():
     from .api.v2 import serializers
 
     param_tools.registry.add(
-        "global", app_settings.GeneralParametersForm, ugettext_lazy("General"))
-    param_tools.registry.add2(
-        "global", "core", ugettext_lazy("General"),
+        "global",
+        "core",
+        gettext_lazy("General"),
         app_settings.GLOBAL_PARAMETERS_STRUCT,
-        serializers.CoreGlobalParametersSerializer)
+        serializers.CoreGlobalParametersSerializer,
+    )
 
 
 class CoreConfig(AppConfig):
     """App configuration."""
 
-    default_auto_field = "django.db.models.BigAutoField"
     name = "modoboa.core"
     verbose_name = "Modoboa core"
 
