@@ -2,10 +2,10 @@
 
 import getpass
 import os
+from os.path import isfile
 import shutil
 import subprocess
 import sys
-from os.path import isfile
 
 import dj_database_url
 
@@ -15,6 +15,7 @@ from django.template import Context, Template
 from django.utils.encoding import smart_str
 
 from modoboa.core.commands import Command
+from modoboa.core.utils import generate_rsa_private_key
 from modoboa.lib.api_client import ModoAPIClient
 from modoboa.lib.sysutils import exec_cmd
 
@@ -35,7 +36,6 @@ DBCONN_TPL = """
 
 
 class DeployCommand(Command):
-
     """The ``deploy`` command."""
 
     help = (  # NOQA:A003
@@ -44,42 +44,54 @@ class DeployCommand(Command):
     )
 
     def __init__(self, *args, **kwargs):
-        super(DeployCommand, self).__init__(*args, **kwargs)
-        self._parser.add_argument("name", type=str,
-                                  help="The name of your Modoboa instance")
+        super().__init__(*args, **kwargs)
         self._parser.add_argument(
-            "--collectstatic", action="store_true", default=False,
-            help="Run django collectstatic command"
+            "name", type=str, help="The name of your Modoboa instance"
         )
         self._parser.add_argument(
-            "--dburl", type=str, nargs="+", default=None,
-            help="A database-url with a name")
-        self._parser.add_argument(
-            "--domain", type=str, default=None,
-            help="The domain under which you want to deploy modoboa")
-        self._parser.add_argument(
-            "--lang", type=str, default="en",
-            help="Set the default language"
+            "--collectstatic",
+            action="store_true",
+            default=False,
+            help="Run django collectstatic command",
         )
         self._parser.add_argument(
-            "--timezone", type=str, default="UTC",
-            help="Set the local timezone"
+            "--dburl",
+            type=str,
+            nargs="+",
+            default=None,
+            help="A database-url with a name",
         )
         self._parser.add_argument(
-            "--devel", action="store_true", default=False,
-            help="Create a development instance"
+            "--domain",
+            type=str,
+            default=None,
+            help="The domain under which you want to deploy modoboa",
         )
         self._parser.add_argument(
-            "--extensions", type=str, nargs="*",
-            help="The list of extension to deploy"
+            "--lang", type=str, default="en", help="Set the default language"
         )
         self._parser.add_argument(
-            "--dont-install-extensions", action="store_true", default=False,
-            help="Do not install extensions using pip"
+            "--timezone", type=str, default="UTC", help="Set the local timezone"
         )
         self._parser.add_argument(
-            "--admin-username", default="admin",
-            help="Username of the initial super administrator"
+            "--devel",
+            action="store_true",
+            default=False,
+            help="Create a development instance",
+        )
+        self._parser.add_argument(
+            "--extensions", type=str, nargs="*", help="The list of extension to deploy"
+        )
+        self._parser.add_argument(
+            "--dont-install-extensions",
+            action="store_true",
+            default=False,
+            help="Do not install extensions using pip",
+        )
+        self._parser.add_argument(
+            "--admin-username",
+            default="admin",
+            help="Username of the initial super administrator",
         )
 
     def _exec_django_command(self, name, cwd, *args):
@@ -102,10 +114,10 @@ class DeployCommand(Command):
         if p.returncode:
             if output:
                 print(
-                    "\n".join([l.decode() for l in output if l is not None]),
-                    file=sys.stderr
+                    "\n".join([line.decode() for line in output if line is not None]),
+                    file=sys.stderr,
                 )
-            print("%s failed, check your configuration" % cmd, file=sys.stderr)
+            print(f"{cmd} failed, check your configuration", file=sys.stderr)
 
     def ask_db_info(self, name="default"):
         """Prompt the user for database information
@@ -115,18 +127,17 @@ class DeployCommand(Command):
 
         :param name: the connection name
         """
-        print("Configuring database connection: %s" % name)
+        print(f"Configuring database connection: {name}")
         info = {
             "conn_name": name,
-            "ENGINE": input(
-                "Database type (mysql, postgres or sqlite3): ")
+            "ENGINE": input("Database type (mysql, postgres or sqlite3): "),
         }
         if info["ENGINE"] not in ["mysql", "postgres", "sqlite3"]:
             raise RuntimeError("Unsupported database engine")
 
         if info["ENGINE"] == "sqlite3":
             info["ENGINE"] = "django.db.backends.sqlite3"
-            info["NAME"] = "%s.db" % name
+            info["NAME"] = f"{name}.db"
             return info
         if info["ENGINE"] == "postgres":
             info["ENGINE"] = "django.db.backends.postgresql"
@@ -135,8 +146,7 @@ class DeployCommand(Command):
             info["ENGINE"] = "django.db.backends.mysql"
             default_port = 3306
         info["HOST"] = input("Database host (default: 'localhost'): ")
-        info["PORT"] = input(
-            "Database port (default: '%s'): " % default_port)
+        info["PORT"] = input(f"Database port (default: '{default_port}'): ")
         # leave port setting empty, if default value is supplied and
         # leave it to django
         if info["PORT"] == default_port:
@@ -154,7 +164,11 @@ class DeployCommand(Command):
         """
         url = "http://api.modoboa.org/"
         official_exts = ModoAPIClient(url).list_extensions()
-        return [extension["name"] for extension in official_exts]
+        return [
+            extension["name"]
+            for extension in official_exts
+            if not extension["deprecated"]
+        ]
 
     def find_extra_settings(self, extensions):
         """Install one or more extensions.
@@ -167,17 +181,15 @@ class DeployCommand(Command):
         for extension in extensions:
             module = __import__(extension[1], locals(), globals(), [])
             basedir = os.path.dirname(module.__file__)
-            if not os.path.exists("{0}/settings.py".format(basedir)):
+            if not os.path.exists(f"{basedir}/settings.py"):
                 continue
             extra_settings.append(extension[1])
         return extra_settings
 
     def handle(self, parsed_args):
         django.setup()
-        management.call_command(
-            "startproject", parsed_args.name, verbosity=False
-        )
-        path = "%(name)s/%(name)s" % {"name": parsed_args.name}
+        management.call_command("startproject", parsed_args.name, verbosity=False)
+        path = f"{parsed_args.name}/{parsed_args.name}"
         sys.path.append(parsed_args.name)
 
         conn_tpl = Template(DBCONN_TPL)
@@ -196,14 +208,12 @@ class DeployCommand(Command):
                 info["conn_name"] = conn_name
                 connections[conn_name] = conn_tpl.render(Context(info))
         else:
-            connections["default"] = conn_tpl.render(
-                Context(self.ask_db_info()))
+            connections["default"] = conn_tpl.render(Context(self.ask_db_info()))
 
         if parsed_args.domain:
             allowed_host = parsed_args.domain
         else:
-            allowed_host = input(
-                "What will be the hostname used to access Modoboa? ")
+            allowed_host = input("What will be the hostname used to access Modoboa? ")
             if not allowed_host:
                 allowed_host = "localhost"
         extra_settings = []
@@ -211,59 +221,50 @@ class DeployCommand(Command):
         if extensions:
             if "all" in extensions:
                 extensions = self._get_extension_list()
-            extensions = [(extension, extension.replace("-", "_"))
-                          for extension in extensions]
+            extensions = [
+                (extension, extension.replace("-", "_")) for extension in extensions
+            ]
             if not parsed_args.dont_install_extensions:
                 cmd = (
-                    sys.executable +
-                    " -m pip install " +
-                    " ".join([extension[0] for extension in extensions])
+                    sys.executable
+                    + " -m pip install "
+                    + " ".join([extension[0] for extension in extensions])
                 )
                 exec_cmd(cmd, capture_output=False)
             extra_settings = self.find_extra_settings(extensions)
             extensions = [extension[1] for extension in extensions]
 
-        bower_components_dir = os.path.realpath(
-            os.path.join(os.path.dirname(__file__), "../../bower_components")
-        )
-
-        mod = __import__(
-            parsed_args.name, globals(), locals(), [smart_str("settings")]
-        )
+        mod = __import__(parsed_args.name, globals(), locals(), [smart_str("settings")])
         tpl = self._render_template(
-            "%s/settings.py.tpl" % self._templates_dir, {
+            f"{self._templates_dir}/settings.py.tpl",
+            {
                 "db_connections": connections,
-                "secret_key": mod.settings.SECRET_KEY,
+                "secret_key": management.utils.get_random_secret_key(),
                 "name": parsed_args.name,
                 "allowed_host": allowed_host,
                 "lang": parsed_args.lang,
                 "timezone": parsed_args.timezone,
-                "bower_components_dir": bower_components_dir,
                 "devmode": parsed_args.devel,
                 "extensions": extensions,
-                "extra_settings": extra_settings
-            }
+                "extra_settings": extra_settings,
+            },
         )
-        with open("%s/settings.py" % path, "w") as fp:
+        with open(f"{path}/settings.py", "w") as fp:
             fp.write(tpl)
-        shutil.copyfile(
-            "%s/urls.py.tpl" % self._templates_dir, "%s/urls.py" % path
-        )
-        os.mkdir("%s/media" % parsed_args.name)
+        generate_rsa_private_key(parsed_args.name)
 
-        if isfile("%s/settings.pyc" % path):
-            os.unlink("%s/settings.pyc" % path)
+        shutil.copyfile(f"{self._templates_dir}/urls.py.tpl", f"{path}/urls.py")
+        os.mkdir(f"{parsed_args.name}/media")
+
+        if isfile(f"{path}/settings.pyc"):
+            os.unlink(f"{path}/settings.pyc")
+        self._exec_django_command("migrate", parsed_args.name, "--noinput")
         self._exec_django_command(
-            "migrate", parsed_args.name, "--noinput"
-        )
-        self._exec_django_command(
-            "load_initial_data", parsed_args.name,
-            "--admin-username", parsed_args.admin_username
+            "load_initial_data",
+            parsed_args.name,
+            "--admin-username",
+            parsed_args.admin_username,
         )
         if parsed_args.collectstatic:
-            self._exec_django_command(
-                "collectstatic", parsed_args.name, "--noinput"
-            )
-        self._exec_django_command(
-            "set_default_site", parsed_args.name, allowed_host
-        )
+            self._exec_django_command("collectstatic", parsed_args.name, "--noinput")
+        self._exec_django_command("set_default_site", parsed_args.name, allowed_host)
