@@ -1,12 +1,12 @@
 """Core application constants."""
 
 from django.conf import settings
-from django.utils.translation import ugettext_lazy
+from django.utils.translation import gettext_lazy
 
-SIMPLEUSERS_ROLE = ("SimpleUsers", ugettext_lazy("Simple user"))
-DOMAINADMINS_ROLE = ("DomainAdmins", ugettext_lazy("Domain administrator"))
-RESELLERS_ROLE = ("Resellers", ugettext_lazy("Reseller"))
-SUPERADMINS_ROLE = ("SuperAdmins", ugettext_lazy("Super administrator"))
+SIMPLEUSERS_ROLE = ("SimpleUsers", gettext_lazy("Simple user"))
+DOMAINADMINS_ROLE = ("DomainAdmins", gettext_lazy("Domain administrator"))
+RESELLERS_ROLE = ("Resellers", gettext_lazy("Reseller"))
+SUPERADMINS_ROLE = ("SuperAdmins", gettext_lazy("Super administrator"))
 
 ROLES = (
     SIMPLEUSERS_ROLE,
@@ -22,24 +22,25 @@ ADMIN_GROUPS = [
 ]
 
 LANGUAGES = (
-    ("br", u"breton"),
-    ("cs", u"čeština"),
-    ("de", u"deutsch"),
-    ("en", u"english"),
-    ("el_GR", u"ελληνικά"),
-    ("es", u"español"),
-    ("fr", u"français"),
-    ("it", u"italiano"),
-    ("ja_JP", u"日本の"),
-    ("nl", u"nederlands"),
-    ("pt_PT", u"português"),
-    ("pt_BR", u"português (BR)"),
-    ("pl_PL", u"polski"),
-    ("ro_RO", u"Română"),
-    ("ru", u"русский"),
-    ("sv", u"svenska"),
-    ("tr_TR", u"türk"),
-    ("zh_TW", u"中文（台灣）"),
+    ("br", "breton"),
+    ("cs", "čeština"),
+    ("de", "deutsch"),
+    ("en", "english"),
+    ("el", "ελληνικά"),
+    ("es", "español"),
+    ("fi", "Suomalainen"),
+    ("fr", "français"),
+    ("it", "italiano"),
+    ("ja", "日本の"),
+    ("nl", "nederlands"),
+    ("pt", "português"),
+    ("pt-br", "português (BR)"),
+    ("pl", "polski"),
+    ("ro", "Română"),
+    ("ru", "русский"),
+    ("sv", "svenska"),
+    ("tr", "türk"),
+    ("zh-hant", "中文（台灣）"),
 )
 
 
@@ -49,9 +50,14 @@ LDAP_GROUP_TYPES = (
 )
 
 LDAP_SECURE_MODES = [
-    ("none", ugettext_lazy("No")),
+    ("none", gettext_lazy("No")),
     ("starttls", "STARTTLS"),
-    ("ssl", "SSL/TLS")
+    ("ssl", "SSL/TLS"),
+]
+
+LDAP_AUTH_METHODS = [
+    ("searchbind", gettext_lazy("Search and bind")),
+    ("directbind", gettext_lazy("Direct bind")),
 ]
 
 PERMISSIONS = {
@@ -72,6 +78,7 @@ PERMISSIONS = {
         ["admin", "senderaddress", "add_senderaddress"],
         ["admin", "senderaddress", "change_senderaddress"],
         ["admin", "senderaddress", "delete_senderaddress"],
+        ["admin", "domainalias", "view_domainalias"],
     ],
     "Resellers": [
         ["core", "user", "add_user"],
@@ -95,13 +102,19 @@ PERMISSIONS = {
         ["admin", "domainalias", "add_domainalias"],
         ["admin", "domainalias", "change_domainalias"],
         ["admin", "domainalias", "delete_domainalias"],
-    ]
+    ],
 }
 
 SMS_BACKENDS = [
-    ("", ugettext_lazy("Choose a provider")),
+    ("", gettext_lazy("Choose a provider")),
     ("ovh", "OVH"),
 ]
 
 if settings.DEBUG:
-    SMS_BACKENDS.insert(1, ("dummy", ugettext_lazy("Dummy")))
+    SMS_BACKENDS.insert(1, ("dummy", gettext_lazy("Dummy")))
+
+TFA_DEVICE_TOKEN_KEY = "otp_device_id"
+TFA_PRE_VERIFY_USER_PK = "tfa_pre_verify_user_pk"
+TFA_PRE_VERIFY_USER_BACKEND = "tfa_pre_verify_user_backend"
+
+DOVEADM_PASS_SCHEME_ALARM = "doveadm_password_scheme_fail"
