@@ -1,14 +1,14 @@
-from django.db import models, migrations
+from django.db import migrations
 
 
 class Migration(migrations.Migration):
 
     dependencies = [
-        ('core', '0001_initial'),
+        ("core", "0001_initial"),
     ]
 
     operations = [
         migrations.DeleteModel(
-            name='Extension',
+            name="Extension",
         ),
     ]
