@@ -5,13 +5,11 @@ Contains weak hashers (the original ones) available with Modoboa.
 """
 
 import base64
-import crypt
 import hashlib
-import string
-from random import Random
+from typing import Optional
 
 from django.utils.crypto import constant_time_compare
-from django.utils.encoding import force_bytes, force_text
+from django.utils.encoding import force_bytes, force_str
 
 
 class MetaHasher(type):
@@ -19,27 +17,39 @@ class MetaHasher(type):
     PasswordHasher Metaclass
     Allow classmethod to be properties
     """
+
     @property
     def name(cls):
         """Returns the name of the hasher"""
-        return cls.__name__.rstrip('Hasher').lower()
+        return cls.__name__.rstrip("Hasher").lower()
 
     @property
     def label(cls):
         """Returns the label of the hasher"""
-        return cls.name if not cls._weak else "{} (weak)".format(cls.name)
+        result = cls.name
+        tags = []
+        if cls._weak:
+            tags.append("weak")
+        if cls.deprecated:
+            tags.append("deprecated")
+        if tags:
+            result += f" ({', '.join(tags)})"
+        return result
 
 
 class PasswordHasher(metaclass=MetaHasher):
     """
     Base class of all hashers.
     """
-    _weak = False
+
+    _weak: bool = False
+    deprecated: bool = False
+    scheme: str
 
     def __init__(self, target="local"):
         self._target = target
 
-    def _encrypt(self, clearvalue, salt=None):
+    def _encrypt(self, clearvalue: str, salt: Optional[str] = None) -> str:  # NOQA
         raise NotImplementedError
 
     def _b64encode(self, pwhash):
@@ -52,7 +62,7 @@ class PasswordHasher(metaclass=MetaHasher):
             return base64.b64encode(pwhash)
         return pwhash
 
-    def encrypt(self, clearvalue):
+    def encrypt(self, clearvalue: str) -> str:
         """Encrypt a password.
 
         The format used to store passwords is the same than dovecot's one::
@@ -65,8 +75,8 @@ class PasswordHasher(metaclass=MetaHasher):
         :rtype: str
         :return: encrypted password
         """
-        pwhash = self._b64encode(self._encrypt(force_text(clearvalue)))
-        return "%s%s" % (self.scheme, force_text(pwhash))
+        pwhash = self._b64encode(self._encrypt(force_str(clearvalue)))
+        return f"{self.scheme}{force_str(pwhash)}"
 
     def verify(self, clearvalue, hashed_value):
         """Verify a password against a hashed value.
@@ -76,8 +86,7 @@ class PasswordHasher(metaclass=MetaHasher):
         :return: True if passwords match, False otherwise
         """
         return constant_time_compare(
-            self._b64encode(self._encrypt(clearvalue, hashed_value)),
-            hashed_value
+            self._b64encode(self._encrypt(clearvalue, hashed_value)), hashed_value
         )
 
     def needs_rehash(self, hashed_value):
@@ -91,15 +100,21 @@ class PasswordHasher(metaclass=MetaHasher):
         return False
 
     @classmethod
-    def get_password_hashers(cls):
+    def get_password_hashers(cls) -> list[type["PasswordHasher"]]:
         """Return all the PasswordHasher supported by Modoboa"""
-        return cls.__subclasses__()
+        return [hasher for hasher in cls.__subclasses__()]
+
+    @classmethod
+    def get_deprecated_password_hashers(cls) -> list[type["PasswordHasher"]]:
+        """Return all the PasswordHasher supported by Modoboa and marked as deprecated."""
+        return [hasher for hasher in cls.__subclasses__() if hasher.deprecated]
 
 
 class PLAINHasher(PasswordHasher):
     """
     Plain (ie. clear) password hasher.
     """
+
     _weak = True
 
     @property
@@ -110,36 +125,13 @@ class PLAINHasher(PasswordHasher):
         return clearvalue
 
 
-class CRYPTHasher(PasswordHasher):
-    """
-    crypt password hasher.
-
-    Uses python `crypt` standard module.
-    """
-    _weak = True
-
-    @property
-    def scheme(self):
-        return "{CRYPT}"
-
-    def _encrypt(self, clearvalue, salt=None):
-        if salt is None:
-            salt = "".join(
-                Random().sample(
-                    string.ascii_letters +
-                    string.digits,
-                    2
-                )
-            )
-        return crypt.crypt(clearvalue, salt)
-
-
 class MD5Hasher(PasswordHasher):
     """
     MD5 password hasher.
 
     Uses python `hashlib` standard module.
     """
+
     _weak = True
 
     @property
@@ -157,6 +149,7 @@ class SHA256Hasher(PasswordHasher):
 
     Uses python `hashlib` and `base64` standard modules.
     """
+
     _weak = True
 
     @property
