@@ -5,17 +5,18 @@ import time
 from email.header import Header
 from email.mime.text import MIMEText
 from email.utils import formataddr, formatdate, getaddresses, make_msgid
+from importlib.metadata import version
 
 import chardet
 import lxml.html
 from lxml.html import defs
-from lxml.html.clean import Cleaner
+from lxml_html_clean import Cleaner
 
 from django.core.mail import EmailMessage
 from django.template.loader import render_to_string
-from django.utils.encoding import smart_str, smart_text
+from django.utils.encoding import smart_str
 from django.utils.html import conditional_escape, escape
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 from modoboa.core import models as core_models
 from modoboa.lib import u2u_decode
@@ -26,20 +27,20 @@ _RE_REMOVE_EXTRA_WHITESPACE = re.compile(r"\n\s*\n")
 _RE_CID = re.compile(r".*[ ]*cid=\"([^\"]*)\".*", re.I)
 
 
-class EmailAddress(object):
+class EmailAddress:
 
     def __init__(self, address):
         self.name, self.address = u2u_decode.decode_address(address)
         if self.name:
-            self.fulladdress = "{} <{}>".format(self.name, self.address)
+            self.fulladdress = f"{self.name} <{self.address}>"
         else:
             self.fulladdress = self.address
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.fulladdress
 
 
-class Email(object):
+class Email:
 
     _basic_headers = (
         "From",
@@ -68,8 +69,7 @@ class Email(object):
             #       updated to use True or False.
             self.links = bool(links)
         elif links == "0":
-            raise TypeError("links == \"0\" is not valid, did you mean True or "
-                            "False?")
+            raise TypeError('links == "0" is not valid, did you mean True or ' "False?")
         else:
             raise TypeError("links should be a boolean value")
 
@@ -102,10 +102,7 @@ class Email(object):
             if self._msg is not None:
                 for header in self._basic_headers:
                     value = self.get_header(self.msg, header)
-                    self._headers.append({
-                        "name": header,
-                        "value": value
-                    })
+                    self._headers.append({"name": header, "value": value})
 
         return self._headers
 
@@ -117,11 +114,11 @@ class Email(object):
     def body(self):
         if self._body is None:
             self._parse(self.msg)
-            self._body = getattr(self, "viewmail_%s" % self.dformat)()
+            self._body = getattr(self, f"viewmail_{self.dformat}")()
 
         return self._body
 
-    def get_header(self, msg, header):
+    def get_header(self, msg, header: str) -> str:
         # msg parameter to maintain compatibility with
         # modoboa_webmail.lib.imapemail.ImapEmail
         if header in msg:
@@ -130,7 +127,7 @@ class Email(object):
                 if not len(value):
                     continue
                 if encoding:
-                    value = smart_text(value, encoding=encoding)
+                    value = smart_str(value, encoding=encoding)
                 elif isinstance(value, bytes):
                     # SMTPUTF8 fallback (most of the time)
                     # Address contains non ASCII chars but is not RFC2047
@@ -173,11 +170,9 @@ class Email(object):
             self._parse_inline_image(msg, level)
 
         if self.contents["plain"]:
-            self.contents["plain"] = self._post_process_plain(
-                self.contents["plain"])
+            self.contents["plain"] = self._post_process_plain(self.contents["plain"])
         if self.contents["html"]:
-            self.contents["html"] = self._post_process_html(
-                self.contents["html"])
+            self.contents["html"] = self._post_process_html(self.contents["html"])
 
     def _parse_text(self, msg, level=0):
         content_type = msg.get_content_subtype()
@@ -221,15 +216,15 @@ class Email(object):
             # Duplicate Content-ID
             return
 
-        self._images[cid] = "data:%s;base64,%s" % (
-            content_type, "".join(msg.get_payload().splitlines(False)))
+        self._images[cid] = "data:{};base64,{}".format(
+            content_type,
+            "".join(msg.get_payload().splitlines(False)),
+        )
 
     def _post_process_plain(self, content):
-        mail_text = (
-            _RE_REMOVE_EXTRA_WHITESPACE.sub("\n\n", content).strip()
-        )
+        mail_text = _RE_REMOVE_EXTRA_WHITESPACE.sub("\n\n", content).strip()
         mail_text = escape(mail_text)
-        return smart_text(mail_text)
+        return smart_str(mail_text)
 
     def _post_process_html(self, content):
         html = lxml.html.fromstring(content)
@@ -249,11 +244,10 @@ class Email(object):
             embedded=True,
             frames=True,
             add_nofollow=True,
-            safe_attrs=safe_attrs
+            safe_attrs=safe_attrs,
         )
-        mail_text = lxml.html.tostring(
-            cleaner.clean_html(html), encoding="unicode")
-        return smart_text(mail_text)
+        mail_text = lxml.html.tostring(cleaner.clean_html(html), encoding="unicode")
+        return smart_str(mail_text)
 
     def _map_cid(self, url):
         if url.startswith("cid:"):
@@ -270,7 +264,7 @@ class Email(object):
             contents = self.contents["plain"]
         else:
             contents = conditional_escape(contents)
-        return "<pre>%s</pre>" % contents
+        return f"<pre>{contents}</pre>"
 
     def viewmail_html(self, contents=None, **kwargs):
         # contents and **kwargs parameters to maintain compatibility with
@@ -288,8 +282,7 @@ class Email(object):
 
 def split_address(address):
     """Split an e-mail address into local part and domain."""
-    assert isinstance(address, str),\
-        "address should be of type str"
+    assert isinstance(address, str), "address should be of type str"
     if "@" not in address:
         local_part = address
         domain = None
@@ -300,27 +293,25 @@ def split_address(address):
 
 def split_local_part(local_part, delimiter=None):
     """Split a local part into local part and extension."""
-    assert isinstance(local_part, str),\
-        "local_part should be of type str"
-    assert isinstance(delimiter, str) or delimiter is None,\
-        "delimiter should be of type str"
+    assert isinstance(local_part, str), "local_part should be of type str"
+    assert (
+        isinstance(delimiter, str) or delimiter is None
+    ), "delimiter should be of type str"
     extension = None
     if local_part.lower() in ["mailer-daemon", "double-bounce"]:
         # never split these special addresses
         pass
-    elif (
-        delimiter == "-" and
-         (local_part.startswith("owner-") or
-          local_part.endswith("-request"))
+    elif delimiter == "-" and (
+        local_part.startswith("owner-") or local_part.endswith("-request")
     ):
         # don't split owner-* or *-request if - is the delimiter, they are
         # special addresses used by mail lists.
         pass
     elif (
-        delimiter and
-        delimiter in local_part and
-        local_part[0] != delimiter and
-        local_part[-1] != delimiter
+        delimiter
+        and delimiter in local_part
+        and local_part[0] != delimiter
+        and local_part[-1] != delimiter
     ):
         local_part, extension = local_part.split(delimiter, 1)
 
@@ -345,8 +336,7 @@ def split_mailbox(mailbox, return_extension=False):
 
 def decode(value_bytes, encoding, append_to_error=""):
     """Try to decode the given string."""
-    assert isinstance(value_bytes, bytes),\
-        "value_bytes should be of type bytes"
+    assert isinstance(value_bytes, bytes), "value_bytes should be of type bytes"
     if len(value_bytes) == 0:
         # short circuit for empty strings
         return ""
@@ -355,13 +345,10 @@ def decode(value_bytes, encoding, append_to_error=""):
     except (UnicodeDecodeError, LookupError):
         encoding = chardet.detect(value_bytes)
         try:
-            value = value_bytes.decode(
-                encoding["encoding"], "replace"
-            )
+            value = value_bytes.decode(encoding["encoding"], "replace")
         except (TypeError, UnicodeDecodeError) as exc:
             raise InternalError(
-                _("unable to determine encoding of string") +
-                append_to_error
+                _("unable to determine encoding of string") + append_to_error
             ) from exc
     return value
 
@@ -393,14 +380,11 @@ def set_email_headers(msg, subject, sender, rcpt):
 
     Subject, From, To, Message-ID, User-Agent, Date
     """
-    import pkg_resources
-
     msg["Subject"] = Header(subject, "utf8")
     msg["From"] = sender
     msg["To"] = prepare_addresses(rcpt)
     msg["Message-ID"] = make_msgid()
-    msg["User-Agent"] = "Modoboa %s" % \
-        (pkg_resources.get_distribution("modoboa").version)
+    msg["User-Agent"] = "Modoboa {}".format(version("modoboa"))
     msg["Date"] = formatdate(time.time(), True)
 
 
@@ -422,12 +406,11 @@ def _sendmail(sender, rcpt, msgstring, server="localhost", port=25):
         s.sendmail(sender, [rcpt], msgstring)
         s.quit()
     except smtplib.SMTPException as e:
-        return False, "SMTP error: %s" % str(e)
+        return False, f"SMTP error: {str(e)}"
     return True, None
 
 
-def sendmail_simple(
-        sender, rcpt, subject="Sample message", content="", **kwargs):
+def sendmail_simple(sender, rcpt, subject="Sample message", content="", **kwargs):
     """Simple way to send a text message
 
     Send a text/plain message with basic headers (msg-id, date).
@@ -460,12 +443,12 @@ def sendmail_fromfile(sender, rcpt, fname):
     """
     try:
         fp = open(fname)
-    except IOError as e:
+    except OSError as e:
         return False, str(e)
 
-    content = """From: %s
-To: %s
-""" % (sender, rcpt)
+    content = f"""From: {sender}
+To: {rcpt}
+"""
     content += fp.read()
     fp.close()
 
@@ -475,11 +458,13 @@ To: %s
 def send_notification(recipient, subject, tpl, sender=None, **kwargs):
     """Send notification by email."""
     if not sender:
-        local_config = (
-            core_models.LocalConfig.objects.select_related("site").first())
+        local_config = core_models.LocalConfig.objects.select_related("site").first()
         sender = local_config.parameters.get_value("sender_address")
     content = render_to_string(tpl, kwargs)
     msg = EmailMessage(
-        subject, content.strip(), sender, [recipient],
+        subject,
+        content.strip(),
+        sender,
+        [recipient],
     )
     msg.send()
