@@ -4,8 +4,7 @@ class _Singleton(type):
 
     def __call__(self, *args, **kwargs):
         if self not in self._instances:
-            self._instances[self] = \
-                super(_Singleton, self).__call__(*args, **kwargs)
+            self._instances[self] = super().__call__(*args, **kwargs)
         return self._instances[self]
 
 
