@@ -5,11 +5,12 @@ This module extra functions/shortcuts to communicate with the system
 
 import inspect
 import os
+import pwd
 import re
 import subprocess
 
 from django.conf import settings
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 
 
 def exec_cmd(cmd, sudo_user=None, pinput=None, capture_output=True, **kwargs):
@@ -26,7 +27,7 @@ def exec_cmd(cmd, sudo_user=None, pinput=None, capture_output=True, **kwargs):
     :return: return code, command output
     """
     if sudo_user is not None:
-        cmd = "sudo -u %s %s" % (sudo_user, cmd)
+        cmd = f"sudo -u {sudo_user} {cmd}"
     kwargs["shell"] = True
     if pinput is not None:
         kwargs["stdin"] = subprocess.PIPE
@@ -42,8 +43,7 @@ def exec_cmd(cmd, sudo_user=None, pinput=None, capture_output=True, **kwargs):
     return process.returncode, output
 
 
-def doveadm_cmd(params, sudo_user=None, pinput=None,
-                capture_output=True, **kwargs):
+def doveadm_cmd(params, pinput=None, capture_output=True, **kwargs):
     """Execute doveadm command.
 
     Run doveadm command using the current user. Set :keyword:`sudo_user` if
@@ -59,24 +59,30 @@ def doveadm_cmd(params, sudo_user=None, pinput=None,
     dpath = None
     code, output = exec_cmd("which doveadm")
     if not code:
-        dpath = force_text(output).strip()
+        dpath = force_str(output).strip()
     else:
         known_paths = getattr(
-            settings, "DOVEADM_LOOKUP_PATH",
-            ("/usr/bin/doveadm", "/usr/local/bin/doveadm")
+            settings,
+            "DOVEADM_LOOKUP_PATH",
+            ("/usr/bin/doveadm", "/usr/local/bin/doveadm"),
         )
         for fpath in known_paths:
             if os.path.isfile(fpath) and os.access(fpath, os.X_OK):
                 dpath = fpath
                 break
+    dovecot_user = getattr(settings, "DOVECOT_USER", "vmail")
+    curuser = pwd.getpwuid(os.getuid()).pw_name
+    sudo_user = dovecot_user if curuser != dovecot_user else None
     if dpath:
-        return exec_cmd("{} {}".format(dpath, params),
-                        sudo_user=sudo_user,
-                        pinput=pinput,
-                        capture_output=capture_output,
-                        **kwargs)
-    else:
-        raise OSError("doveadm command not found")
+        return exec_cmd(
+            f"{dpath} {params}",
+            sudo_user=sudo_user,
+            pinput=pinput,
+            capture_output=capture_output,
+            **kwargs,
+        )
+
+    raise OSError("doveadm command not found")
 
 
 def guess_extension_name():
