@@ -1,19 +1,19 @@
-from django.db import models, migrations
+from django.db import migrations
 
 
 class Migration(migrations.Migration):
 
     dependencies = [
-        ('limits', '0001_initial'),
+        ("limits", "0001_initial"),
     ]
 
     operations = [
         migrations.AlterModelTable(
-            name='limit',
+            name="limit",
             table=None,
         ),
         migrations.AlterModelTable(
-            name='limitspool',
+            name="limitspool",
             table=None,
         ),
     ]
