@@ -1,15 +1,14 @@
 """Classes to define graphics."""
 
+import datetime
+import json
 import inspect
 from itertools import chain
 import os
 
-from lxml import etree
-import six
-
 from django.conf import settings
-from django.utils.encoding import smart_bytes, smart_text
-from django.utils.translation import ugettext as _, ugettext_lazy
+from django.utils.encoding import smart_bytes, smart_str
+from django.utils.translation import gettext as _, gettext_lazy
 
 from modoboa.admin import models as admin_models
 from modoboa.lib import exceptions
@@ -18,15 +17,13 @@ from modoboa.parameters import tools as param_tools
 
 
 class Curve:
-
     """Graphic curve.
 
     Simple way to represent a graphic curve.
     """
 
     def __init__(self, dsname, color, legend, cfunc="AVERAGE"):
-        """Constructor.
-        """
+        """Constructor."""
         self.dsname = dsname
         self.color = color
         self.legend = legend
@@ -39,13 +36,12 @@ class Curve:
         :return: a list
         """
         rrdfile = os.path.join(
-            param_tools.get_global_parameter("rrd_rootdir"), "%s.rrd" % rrdfile
+            param_tools.get_global_parameter("rrd_rootdir"), f"{rrdfile}.rrd"
         )
         return [
-            'DEF:%s=%s:%s:%s' %
-            (self.dsname, rrdfile, self.dsname, self.cfunc),
-            'CDEF:%(ds)spm=%(ds)s,UN,0,%(ds)s,IF,60,*' % {"ds": self.dsname},
-            'XPORT:%spm:"%s"' % (self.dsname, self.legend)
+            f"DEF:{self.dsname}={rrdfile}:{self.dsname}:{self.cfunc}",
+            f"CDEF:{self.dsname}pm={self.dsname},UN,0,{self.dsname},IF,60,*",
+            f'XPORT:{self.dsname}pm:"{self.legend}"',
         ]
 
 
@@ -56,7 +52,7 @@ class Graphic:
         """Constructor."""
         self._curves = []
         try:
-            order = getattr(self, "order")
+            order = self.order
         except AttributeError:
             for member in inspect.getmembers(self):
                 if isinstance(member[1], Curve):
@@ -84,52 +80,47 @@ class Graphic:
             dpath = output.strip()
         else:
             known_paths = getattr(
-                settings, "RRDTOOL_LOOKUP_PATH",
-                ("/usr/bin/rrdtool", "/usr/local/bin/rrdtool")
+                settings,
+                "RRDTOOL_LOOKUP_PATH",
+                ("/usr/bin/rrdtool", "/usr/local/bin/rrdtool"),
             )
             for fpath in known_paths:
                 if os.path.isfile(fpath) and os.access(fpath, os.X_OK):
                     dpath = fpath
         if dpath is None:
-            raise exceptions.InternalError(
-                _("Failed to locate rrdtool binary."))
-        return smart_text(dpath)
+            raise exceptions.InternalError(_("Failed to locate rrdtool binary."))
+        return smart_str(dpath)
 
     def export(self, rrdfile, start, end):
-        """Export data to XML using rrdtool and convert it to JSON."""
+        """Export data to JSON using rrdtool."""
         result = []
         cmdargs = []
         for curve in self._curves:
-            result += [{
-                "name": str(curve.legend),
-                "color": curve.color, "data": []
-            }]
+            result += [
+                {"name": str(curve.legend), "backgroundColor": curve.color, "data": []}
+            ]
             cmdargs += curve.to_rrd_command_args(rrdfile)
         code = 0
 
-        cmd = "{} xport --start {} --end {} ".format(
-            self.rrdtool_binary, str(start), str(end))
+        cmd = f"{self.rrdtool_binary} xport --json -t --start {str(start)} --end {str(end)} "
         cmd += " ".join(cmdargs)
         code, output = exec_cmd(smart_bytes(cmd))
         if code:
             return []
 
-        tree = etree.fromstring(output)
-        timestamp = int(tree.xpath('/xport/meta/start')[0].text)
-        step = int(tree.xpath('/xport/meta/step')[0].text)
-        for row in tree.xpath('/xport/data/row'):
-            for vindex, value in enumerate(row.findall('v')):
-                if value.text == 'NaN':
-                    result[vindex]['data'].append({'x': timestamp, 'y': 0})
-                else:
-                    result[vindex]['data'].append(
-                        {'x': timestamp, 'y': float(value.text)}
-                    )
-            timestamp += step
+        xport = json.loads(output)
+        for row in xport["data"]:
+            timestamp = int(row[0])
+            date = datetime.datetime.fromtimestamp(timestamp).isoformat(sep=" ")
+            for vindex, value in enumerate(row[1:]):
+                result[vindex]["data"].append(
+                    {"x": date, "y": value, "timestamp": timestamp}
+                )
+
         return result
 
 
-class GraphicSet(object):
+class GraphicSet:
     """A set of graphics."""
 
     domain_selector = False
@@ -154,42 +145,49 @@ class GraphicSet(object):
     def get_graphic_names(self):
         return [graphic.display_name for graphic in self._graphics]
 
-    def get_file_name(self, request):
+    def get_file_name(self, request, searchq):
         """Return database file name."""
         return self.file_name
 
-    def export(self, rrdfile, start, end):
+    def export(self, rrdfile, start, end, graphic=None):
         result = {}
         for graph in self.graphics:
-            result[graph.display_name] = {
-                "title": six.text_type(graph.title),
-                "curves": graph.export(rrdfile, start, end)
-            }
+            if graphic is None or graphic == graph.display_name:
+                result[graph.display_name] = {
+                    "title": str(graph.title),
+                    "series": graph.export(rrdfile, start, end),
+                }
         return result
 
 
 class AverageTraffic(Graphic):
     """Average traffic."""
 
-    title = ugettext_lazy('Average traffic (msgs/min)')
+    title = gettext_lazy("Average traffic (msgs/min)")
 
     # Curve definitions
-    sent = Curve("sent", "lawngreen", ugettext_lazy("sent messages"))
-    recv = Curve("recv", "steelblue", ugettext_lazy("received messages"))
-    bounced = Curve("bounced", "yellow", ugettext_lazy("bounced messages"))
-    reject = Curve("reject", "tomato", ugettext_lazy("rejected messages"))
-    virus = Curve("virus", "orange", ugettext_lazy("virus messages"))
-    spam = Curve("spam", "silver", ugettext_lazy("spam messages"))
+    sent = Curve("sent", "lawngreen", gettext_lazy("sent messages"))
+    recv = Curve("recv", "steelblue", gettext_lazy("received messages"))
+    bounced = Curve("bounced", "yellow", gettext_lazy("bounced messages"))
+    reject = Curve("reject", "tomato", gettext_lazy("rejected messages"))
+    virus = Curve("virus", "orange", gettext_lazy("virus messages"))
+    spam = Curve("spam", "silver", gettext_lazy("spam messages"))
 
     order = ["reject", "bounced", "recv", "sent", "virus", "spam"]
 
     def __init__(self, greylist=False):
         if greylist:
             self.greylist = Curve(
-                "greylist", "dimgrey", ugettext_lazy("greylisted messages"))
+                "greylist", "dimgrey", gettext_lazy("greylisted messages")
+            )
             self.order = [
-                "reject", "greylist", "bounced", "recv", "sent", "virus",
-                "spam"
+                "reject",
+                "greylist",
+                "bounced",
+                "recv",
+                "sent",
+                "virus",
+                "spam",
             ]
         super().__init__()
 
@@ -197,20 +195,18 @@ class AverageTraffic(Graphic):
 class AverageTrafficSize(Graphic):
     """Average traffic size."""
 
-    title = ugettext_lazy('Average normal traffic size (bytes/min)')
+    title = gettext_lazy("Average normal traffic size (bytes/min)")
 
     # Curve definitions
-    size_recv = Curve("size_recv", "orange", ugettext_lazy("received size"))
-    size_sent = Curve(
-        "size_sent", "mediumturquoise", ugettext_lazy("sent size")
-    )
+    size_recv = Curve("size_recv", "orange", gettext_lazy("received size"))
+    size_sent = Curve("size_sent", "mediumturquoise", gettext_lazy("sent size"))
 
 
 class MailTraffic(GraphicSet):
     """Mail traffic graphic set."""
 
     domain_selector = True
-    title = ugettext_lazy("Mail traffic")
+    title = gettext_lazy("Mail traffic")
     _graphics = [AverageTraffic, AverageTrafficSize]
 
     def __init__(self, greylist=False):
@@ -236,8 +232,7 @@ class MailTraffic(GraphicSet):
         results = list(
             chain(
                 admin_models.Domain.objects.filter(name__startswith=pattern),
-                admin_models.DomainAlias.objects.filter(
-                    name__startswith=pattern)
+                admin_models.DomainAlias.objects.filter(name__startswith=pattern),
             )
         )
         if len(results) != 1:
@@ -246,24 +241,22 @@ class MailTraffic(GraphicSet):
             raise exceptions.PermDeniedException
         return results[0].name
 
-    def get_file_name(self, request):
+    def get_file_name(self, user, searchq):
         """Retrieve file name according to user and args."""
-        searchq = request.GET.get("searchquery", None)
-        return self._check_domain_access(request.user, searchq)
+        return self._check_domain_access(user, searchq)
 
 
 class AccountCreationGraphic(Graphic):
     """Account creation over time."""
 
-    title = ugettext_lazy("Average account creation (account/hour)")
+    title = gettext_lazy("Average account creation (account/hour)")
 
-    accounts = Curve(
-        "new_accounts", "steelblue", ugettext_lazy("New accounts"))
+    accounts = Curve("new_accounts", "steelblue", gettext_lazy("New accounts"))
 
 
 class AccountGraphicSet(GraphicSet):
     """A graphic set for accounts."""
 
     file_name = "new_accounts"
-    title = ugettext_lazy("Accounts")
+    title = gettext_lazy("Accounts")
     _graphics = [AccountCreationGraphic]
