@@ -1,5 +1,3 @@
-# coding: utf-8
-
 """
 Postfix log parser.
 
@@ -19,7 +17,6 @@ Predefined events are:
 """
 
 from datetime import datetime
-import io
 import os
 import re
 import sys
@@ -43,8 +40,16 @@ from ... import models
 rrdstep = 60
 xpoints = 540
 points_per_sample = 3
-variables = ["sent", "recv", "bounced", "reject", "spam", "virus",
-             "size_sent", "size_recv"]
+variables = [
+    "sent",
+    "recv",
+    "bounced",
+    "reject",
+    "spam",
+    "virus",
+    "size_sent",
+    "size_recv",
+]
 
 
 class LogParser:
@@ -73,10 +78,9 @@ class LogParser:
         # set up regular expression
         self._date_expressions = [
             r"(?P<month>\w+)\s+(?P<day>\d+)\s+(?P<hour>\d+):(?P<min>\d+):(?P<sec>\d+)(?P<eol>.*)",  # noqa
-            r"(?P<year>\d+)-(?P<month>\d+)-(?P<day>\d+)T(?P<hour>\d+):(?P<min>\d+):(?P<sec>\d+)\.\d+\+\d+:\d+(?P<eol>.*)"  # noqa
+            r"(?P<year>\d+)-(?P<month>\d+)-(?P<day>\d+)T(?P<hour>\d+):(?P<min>\d+):(?P<sec>\d+)\.\d+[\+\-]\d+:\d+(?P<eol>.*)",  # noqa
         ]
-        self._date_expressions = [
-            re.compile(v) for v in self._date_expressions]
+        self._date_expressions = [re.compile(v) for v in self._date_expressions]
         self.date_expr = None
         self._regex = {
             "line": r"\s+([-\w\.]+)\s+(\w+)/?(\w*)[[](\d+)[]]:\s+(.*)",
@@ -91,14 +95,13 @@ class LogParser:
             "rmilter_msg_done": r"msg done: queue_id: <(?P<queue_id>[^>]+)>; message id: <(?P<message_id>[^>]+)>.*; from: <(?P<from>[^>]+)>; rcpt: <(?P<rcpt>[^>]+)>.*; spam scan: (?P<action>[^;]+); virus scan:",  # noqa
             "rmilter_spam": r"mlfi_eom: (rejecting spam|add spam header to message according to spamd action)",  # noqa
             "rmilter_virus": r"mlfi_eom:.* virus found",
-            "rmilter_greylist": (
-                r"GREYLIST\([0-9]+\.[0-9]{2}\)\[greylisted[^\]]*\]")
+            "rmilter_greylist": (r"GREYLIST\([0-9]+\.[0-9]{2}\)\[greylisted[^\]]*\]"),
         }
         self._regex = {k: re.compile(v) for k, v in self._regex.items()}
         self._srs_regex = {
             "detect_srs": "^SRS[01][+-=]",
             "reverse_srs0": r"^SRS0[+-=]\S+=\S{2}=(\S+)=(\S+)\@\S+$",
-            "reverse_srs1": r"^SRS1[+-=]\S+=\S+==\S+=\S{2}=(\S+)=(\S+)\@\S+$"
+            "reverse_srs1": r"^SRS1[+-=]\S+=\S+==\S+=\S{2}=(\S+)=(\S+)\@\S+$",
         }
         self._srs_regex = {
             k: re.compile(v, re.IGNORECASE) for k, v in self._srs_regex.items()
@@ -192,7 +195,7 @@ class LogParser:
         """
         ds_type = "ABSOLUTE"
         rows = xpoints / points_per_sample
-        realrows = int(rows * 1.1)    # ensure that the full range is covered
+        realrows = int(rows * 1.1)  # ensure that the full range is covered
         day_steps = int(3600 * 24 / (rrdstep * rows))
         week_steps = day_steps * 7
         month_steps = week_steps * 5
@@ -201,18 +204,15 @@ class LogParser:
         # Set up data sources for our RRD
         params = []
         for v in variables:
-            params += ["DS:%s:%s:%s:0:U" % (v, ds_type, rrdstep * 2)]
+            params += [f"DS:{v}:{ds_type}:{rrdstep * 2}:0:U"]
 
         # Set up RRD to archive data
         for cf in ["AVERAGE", "MAX"]:
             for step in [day_steps, week_steps, month_steps, year_steps]:
-                params += ["RRA:%s:0.5:%s:%s" % (cf, step, realrows)]
+                params += [f"RRA:{cf}:0.5:{step}:{realrows}"]
 
         # With those setup, we can now created the RRD
-        rrdtool.create(str(fname),
-                       "--start", str(m),
-                       "--step", str(rrdstep),
-                       *params)
+        rrdtool.create(str(fname), "--start", str(m), "--step", str(rrdstep), *params)
         return m
 
     def add_datasource_to_rrd(self, fname, dsname):
@@ -221,16 +221,16 @@ class LogParser:
         Add missing Data Sources (DS) to existing Round Robin Archive (RRA):
         See init_rrd for details.
         """
-        ds_def = "DS:%s:ABSOLUTE:%s:0:U" % (dsname, rrdstep * 2)
+        ds_def = f"DS:{dsname}:ABSOLUTE:{rrdstep * 2}:0:U"
         rrdtool.tune(fname, ds_def)
-        self._dprint("[rrd] added DS %s to %s" % (dsname, fname))
+        self._dprint(f"[rrd] added DS {dsname} to {fname}")
 
     def add_point_to_rrd(self, fname, tpl, values, ts=None):
         """Try to add a new point to RRD file."""
         if ts:
-            values = "{}:{}".format(ts, values)
+            values = f"{ts}:{values}"
         if self.verbose:
-            print("[rrd] VERBOSE update -t %s %s" % (tpl, values))
+            print(f"[rrd] VERBOSE update -t {tpl} {values}")
         try:
             rrdtool.update(str(fname), "-t", tpl, values)
         except rrdtool.OperationalError as e:
@@ -249,20 +249,20 @@ class LogParser:
         False : syslog may have probably been already recorded
         or something wrong
         """
-        fname = "%s/%s.rrd" % (self.workdir, dom)
+        fname = f"{self.workdir}/{dom}.rrd"
         m = t - (t % rrdstep)
 
-        self._dprint("[rrd] updating %s" % fname)
+        self._dprint(f"[rrd] updating {fname}")
         if not os.path.exists(fname):
             self.lupdates[fname] = self.init_rrd(fname, m)
-            self._dprint("[rrd] create new RRD file %s" % fname)
+            self._dprint(f"[rrd] create new RRD file {fname}")
         else:
             if fname not in self.lupdates:
                 self.lupdates[fname] = rrdtool.last(str(fname))
 
         if m <= self.lupdates[fname]:
             if self.verbose:
-                print("[rrd] VERBOSE events at %s already recorded in RRD" % m)
+                print(f"[rrd] VERBOSE events at {m} already recorded in RRD")
             return False
 
         tpl = ""
@@ -274,14 +274,14 @@ class LogParser:
         # Est ce vraiment nécessaire... ?
         if m > self.lupdates[fname] + rrdstep:
             values = ""
-            for v in variables:
+            for _v in variables:
                 if values != "":
                     values += ":"
                 values += "0"
             for p in range(self.lupdates[fname] + rrdstep, m, rrdstep):
                 self.add_point_to_rrd(fname, tpl, values, ts=p)
 
-        values = "%s" % m
+        values = f"{m}"
         tpl = ""
         for v in variables:
             values += ":"
@@ -326,14 +326,13 @@ class LogParser:
         :param month: the month of the current record beeing parsed
         :return: an integer
         """
-        month = time.strptime(month, "%b").tm_mon if not month.isdigit() \
-            else int(month)
+        month = time.strptime(month, "%b").tm_mon if not month.isdigit() else int(month)
         if self.curmonth == 1 and month != self.curmonth:
             return self.__year - 1
         return self.__year
 
     def is_srs_forward(self, mail_address):
-        """ Check if mail address has been mangled by SRS
+        """Check if mail address has been mangled by SRS
 
         Sender Rewriting Scheme (SRS) modifies mail adresses so that they
         always end in a local domain.
@@ -344,7 +343,7 @@ class LogParser:
         return self._srs_regex["detect_srs"].match(mail_address) is not None
 
     def reverse_srs(self, mail_address):
-        """ Try to unwind a mail address rewritten by SRS
+        """Try to unwind a mail address rewritten by SRS
 
         Sender Rewriting Scheme (SRS) modifies mail adresses so that they
         always end in a local domain. Common Postfix implementations of SRS
@@ -354,16 +353,15 @@ class LogParser:
         :return a str
         """
         m = self._srs_regex["reverse_srs0"].match(mail_address)
-        m = self._srs_regex["reverse_srs1"].match(mail_address) \
-            if m is None else m
+        m = self._srs_regex["reverse_srs1"].match(mail_address) if m is None else m
 
         if m is not None:
-            return "%s@%s" % m.group(2, 1)
+            return "{}@{}".format(*m.group(2, 1))
         else:
             return mail_address
 
     def _parse_amavis(self, log, host, pid, subprog):
-        """ Parse an Amavis log entry.
+        """Parse an Amavis log entry.
 
         :param str log: logged message
         :param str host: hostname
@@ -384,7 +382,7 @@ class LogParser:
         return False
 
     def _parse_rmilter(self, log, host, pid, subprog):
-        """ Parse an Rmilter log entry.
+        """Parse an Rmilter log entry.
 
         :param str log: logged message
         :param str host: hostname
@@ -400,24 +398,18 @@ class LogParser:
         # Virus check must come before spam check due to pattern similarity.
         m = self._regex["rmilter_virus"].match(msg)
         if m is not None:
-            self.workdict[workdict_key] = {
-                "action": "virus"
-            }
+            self.workdict[workdict_key] = {"action": "virus"}
             return True
         m = self._regex["rmilter_spam"].match(msg)
         if m is not None:
-            self.workdict[workdict_key] = {
-                "action": "spam"
-            }
+            self.workdict[workdict_key] = {"action": "spam"}
             return True
 
         # Greylisting
         if self.greylist:
             m = self._regex["rmilter_greylist"].search(msg)
             if m is not None:
-                self.workdict[workdict_key] = {
-                    "action": "greylist"
-                }
+                self.workdict[workdict_key] = {"action": "greylist"}
                 return True
 
         # Gather information about message sender and queue ID
@@ -434,7 +426,7 @@ class LogParser:
         return False
 
     def _parse_postfix(self, log, host, pid, subprog):
-        """ Parse a log entry generated by Postfix.
+        """Parse a log entry generated by Postfix.
 
         :param str log: logged message
         :param str host: hostname
@@ -452,10 +444,8 @@ class LogParser:
             m = self._regex["reject"].match(msg)
             dom = m.group(1) if m is not None else None
             if dom in self.domains:
-                condition = (
-                    self.greylist and (
-                        "Greylisted" in msg or
-                        (subprog == "postscreen" and " 450 " in msg))
+                condition = self.greylist and (
+                    "Greylisted" in msg or (subprog == "postscreen" and " 450 " in msg)
                 )
                 if condition:
                     self.inc_counter(dom, "greylist")
@@ -474,7 +464,7 @@ class LogParser:
         if m is not None:
             self.workdict[queue_id] = {
                 "from": self.reverse_srs(m.group(1)),
-                "size": int(m.group(2))
+                "size": int(m.group(2)),
             }
             return True
 
@@ -484,12 +474,10 @@ class LogParser:
             return False
         (msg_to, msg_status) = m.groups()
         if queue_id not in self.workdict:
-            self._dprint("[parser] inconsistent mail (%s: %s), skipping"
-                         % (queue_id, msg_to))
+            self._dprint(f"[parser] inconsistent mail ({queue_id}: {msg_to}), skipping")
             return True
         if msg_status not in variables:
-            self._dprint("[parser] unsupported status %s, skipping"
-                         % msg_status)
+            self._dprint(f"[parser] unsupported status {msg_status}, skipping")
             return True
 
         # orig_to is optional.
@@ -500,15 +488,11 @@ class LogParser:
         from_domain = split_mailbox(self.workdict[queue_id]["from"])[1]
         if from_domain is not None and from_domain in self.domains:
             self.inc_counter(from_domain, "sent")
-            self.inc_counter(from_domain, "size_sent",
-                             self.workdict[queue_id]["size"])
+            self.inc_counter(from_domain, "size_sent", self.workdict[queue_id]["size"])
 
         # Handle local "to" domains.
         to_domain = None
-        condition = (
-            msg_orig_to is not None and
-            not self.is_srs_forward(msg_orig_to)
-        )
+        condition = msg_orig_to is not None and not self.is_srs_forward(msg_orig_to)
         if condition:
             to_domain = split_mailbox(msg_orig_to)[1]
         if to_domain is None:
@@ -516,35 +500,33 @@ class LogParser:
 
         if msg_status == "sent":
             self.inc_counter(to_domain, "recv")
-            self.inc_counter(to_domain, "size_recv",
-                             self.workdict[queue_id]["size"])
+            self.inc_counter(to_domain, "size_recv", self.workdict[queue_id]["size"])
         else:
             self.inc_counter(to_domain, msg_status)
 
         last_message = models.Maillog.objects.last()
         cur_dt = datetime.fromtimestamp(self.orig_ts)
         tz = timezone.get_current_timezone()
-        cur_dt = tz.localize(cur_dt)
-        condition = (
-            last_message and (
-                cur_dt < last_message.date or
-                (cur_dt == last_message.date and
-                 models.Maillog.objects.filter(
-                     date=cur_dt, queue_id=queue_id).exists())
+        cur_dt = cur_dt.replace(tzinfo=tz)
+        condition = last_message and (
+            cur_dt < last_message.date
+            or (
+                cur_dt == last_message.date
+                and models.Maillog.objects.filter(
+                    date=cur_dt, queue_id=queue_id
+                ).exists()
             )
         )
         if not condition:
             if from_domain in self.domains:
                 from_domain = Domain.objects.filter(
-                    Q(name=from_domain) |
-                    Q(domainalias__name=from_domain)
+                    Q(name=from_domain) | Q(domainalias__name=from_domain)
                 ).first()
             else:
                 from_domain = None
             if to_domain in self.domains:
                 to_domain = Domain.objects.filter(
-                    Q(name=to_domain) |
-                    Q(domainalias__name=to_domain)
+                    Q(name=to_domain) | Q(domainalias__name=to_domain)
                 ).first()
             else:
                 to_domain = None
@@ -559,7 +541,7 @@ class LogParser:
                 size=self.workdict[queue_id]["size"],
                 status=msg_status,
                 from_domain=from_domain,
-                to_domain=to_domain
+                to_domain=to_domain,
             )
 
         return True
@@ -578,12 +560,11 @@ class LogParser:
         host, prog, subprog, pid, log = m.groups()
 
         try:
-            parser = getattr(self, "_parse_{}".format(prog))
+            parser = getattr(self, f"_parse_{prog}")
             if not parser(log, host, pid, subprog):
-                self._dprint("[parser] ignoring %r log: %r" % (prog, log))
+                self._dprint(f"[parser] ignoring {prog!r} log: {log!r}")
         except AttributeError:
-            self._dprint(
-                "[parser] no log handler for \"{}\": {}".format(prog, log))
+            self._dprint(f'[parser] no log handler for "{prog}": {log}')
 
     def process(self):
         """Process the log file.
@@ -595,12 +576,12 @@ class LogParser:
             with open(self.logfile, encoding="utf-8", errors="ignore") as fp:
                 for line in fp:
                     self._parse_line(line)
-        except IOError as errno:
-            self._dprint("%s" % errno)
+        except OSError as errno:
+            self._dprint(f"{errno}")
             sys.exit(1)
 
         for dom, data in self.data.items():
-            self._dprint("[rrd] dealing with domain %s" % dom)
+            self._dprint(f"[rrd] dealing with domain {dom}")
             for t in sorted(data.keys()):
                 self.update_rrd(dom, t)
 
@@ -611,14 +592,21 @@ class Command(BaseCommand):
     def add_arguments(self, parser):
         """Add extra arguments to command line."""
         parser.add_argument(
-            "--logfile", default=None,
-            help="postfix log in syslog format", metavar="FILE")
+            "--logfile",
+            default=None,
+            help="postfix log in syslog format",
+            metavar="FILE",
+        )
         parser.add_argument(
-            "--verbose", default=False, action="store_true",
-            dest="verbose", help="Set verbose mode")
+            "--verbose",
+            default=False,
+            action="store_true",
+            dest="verbose",
+            help="Set verbose mode",
+        )
         parser.add_argument(
-            "--debug", default=False, action="store_true",
-            help="Set debug mode")
+            "--debug", default=False, action="store_true", help="Set debug mode"
+        )
 
     def can_start(self):
         """Check if another process is already running or not."""
@@ -643,11 +631,10 @@ class Command(BaseCommand):
             sys.exit(2)
         if options["logfile"] is None:
             options["logfile"] = param_tools.get_global_parameter(
-                "logfile", app="maillog")
-        greylist = param_tools.get_global_parameter(
-            "greylist", raise_exception=False)
+                "logfile", app="maillog"
+            )
+        greylist = param_tools.get_global_parameter("greylist", raise_exception=False)
         p = LogParser(
-            options, param_tools.get_global_parameter("rrd_rootdir"),
-            None, greylist
+            options, param_tools.get_global_parameter("rrd_rootdir"), None, greylist
         )
         p.process()
