@@ -1,24 +1,22 @@
 """Policy daemon related tests."""
 
 import asyncio
-# from mock import patch
-from asynctest import patch
+from aiosmtplib import send
+from unittest.mock import AsyncMock
 from multiprocessing import Process
 import socket
 import time
 
-import redis
 
 from django import db
-from django.conf import settings
 from django.core.management import call_command
 from django.test import TransactionTestCase
-from django.urls import reverse
 
 from modoboa.admin import factories as admin_factories
 from modoboa.admin import models as admin_models
 from modoboa.core import models as core_models
-from modoboa.lib.tests import ModoTestCase, ParametersMixin
+from modoboa.lib.redis import get_redis_connection
+from modoboa.lib.tests import ModoAPITestCase, ParametersMixin
 from modoboa.policyd import core as policyd_core
 
 from . import constants
@@ -33,17 +31,11 @@ class RedisTestCaseMixin:
 
     def setUp(self):
         super().setUp()
-        self.rclient = redis.Redis(
-            host=settings.REDIS_HOST,
-            port=settings.REDIS_PORT,
-            db=settings.REDIS_QUOTA_DB
-        )
-        self.rclient.set_response_callback("HGET", int)
+        self.rclient = get_redis_connection()
         self.rclient.delete(constants.REDIS_HASHNAME)
 
 
-class PolicyDaemonTestCase(
-        RedisTestCaseMixin, ParametersMixin, TransactionTestCase):
+class PolicyDaemonTestCase(RedisTestCaseMixin, ParametersMixin, TransactionTestCase):
     """Test cases for policy daemon.
 
     A redis instance is required to run those tests.
@@ -57,8 +49,7 @@ class PolicyDaemonTestCase(
         self.admin.role = "SuperAdmins"
 
         db.connections.close_all()
-        patcher = patch("aiosmtplib.send")
-        self.send_mock = patcher.start()
+        self.send_mock = AsyncMock(send)
         self.process = Process(target=start_policy_daemon)
         self.process.daemon = True
         self.process.start()
@@ -72,7 +63,7 @@ class PolicyDaemonTestCase(
         domain.save()
         self.assertEqual(
             self.rclient.hget(constants.REDIS_HASHNAME, domain.name),
-            domain.message_limit
+            domain.message_limit,
         )
         return domain
 
@@ -83,8 +74,7 @@ class PolicyDaemonTestCase(
         mb.message_limit = value
         mb.save()
         self.assertEqual(
-            self.rclient.hget(constants.REDIS_HASHNAME, account.email),
-            mb.message_limit
+            self.rclient.hget(constants.REDIS_HASHNAME, account.email), mb.message_limit
         )
         return account
 
@@ -109,25 +99,29 @@ class PolicyDaemonTestCase(
         s = self.connect_to_daemon()
 
         # This one will be accepted
-        s.send(b"""protocol_state=RCPT
+        s.send(
+            b"""protocol_state=RCPT
 sasl_username=user@test.com
 
-""")
+"""
+        )
         res = s.recv(1024)
         self.assertEqual(res, b"action=dunno\n\n")
         self.assertEqual(
             self.rclient.hget(constants.REDIS_HASHNAME, domain.name),
-            domain.message_limit - 1
+            domain.message_limit - 1,
         )
         s.close()
 
         # This one will be accepted and will trigger a notification
         # but I can't check it yet
         s = self.connect_to_daemon()
-        s.send(b"""protocol_state=RCPT
+        s.send(
+            b"""protocol_state=RCPT
 sasl_username=user@test.com
 
-""")
+"""
+        )
         time.sleep(0.1)
         res = s.recv(1024)
         self.assertEqual(res, b"action=dunno\n\n")
@@ -135,10 +129,12 @@ sasl_username=user@test.com
 
         # This one will be denied
         s = self.connect_to_daemon()
-        s.send(b"""protocol_state=RCPT
+        s.send(
+            b"""protocol_state=RCPT
 sasl_username=user@test.com
 
-""")
+"""
+        )
         res = s.recv(1024)
         self.assertEqual(
             res, b"action=defer_if_permit Daily limit reached, retry later\n\n"
@@ -151,25 +147,29 @@ sasl_username=user@test.com
         s = self.connect_to_daemon()
 
         # This one will be accepted
-        s.send(b"""protocol_state=RCPT
+        s.send(
+            b"""protocol_state=RCPT
 sasl_username=user@test.com
 
-""")
+"""
+        )
         res = s.recv(1024)
         self.assertEqual(res, b"action=dunno\n\n")
         self.assertEqual(
             self.rclient.hget(constants.REDIS_HASHNAME, account.email),
-            account.mailbox.message_limit - 1
+            account.mailbox.message_limit - 1,
         )
         s.close()
 
         # This one will be accepted and will trigger a notification
         # but I can't check it yet
         s = self.connect_to_daemon()
-        s.send(b"""protocol_state=RCPT
+        s.send(
+            b"""protocol_state=RCPT
 sasl_username=user@test.com
 
-""")
+"""
+        )
         time.sleep(0.1)
         res = s.recv(1024)
         self.assertEqual(res, b"action=dunno\n\n")
@@ -177,10 +177,12 @@ sasl_username=user@test.com
 
         # This one will be denied
         s = self.connect_to_daemon()
-        s.send(b"""protocol_state=RCPT
+        s.send(
+            b"""protocol_state=RCPT
 sasl_username=user@test.com
 
-""")
+"""
+        )
         res = s.recv(1024)
         self.assertEqual(
             res, b"action=defer_if_permit Daily limit reached, retry later\n\n"
@@ -200,17 +202,14 @@ sasl_username=user@test.com
             await policyd_core.reset_counters()
 
         # Run the async test
-        coro = asyncio.coroutine(run_test)
-        event_loop.run_until_complete(coro())
+        event_loop.run_until_complete(run_test())
         event_loop.close()
 
-        self.assertEqual(
-            self.rclient.hget(constants.REDIS_HASHNAME, domain.name), 20)
-        self.assertEqual(
-            self.rclient.hget(constants.REDIS_HASHNAME, account.email), 10)
+        self.assertEqual(self.rclient.hget(constants.REDIS_HASHNAME, domain.name), 20)
+        self.assertEqual(self.rclient.hget(constants.REDIS_HASHNAME, account.email), 10)
 
 
-class ModelsTestCase(RedisTestCaseMixin, ModoTestCase):
+class ModelsTestCase(RedisTestCaseMixin, ModoAPITestCase):
     """Admin models test cases."""
 
     @classmethod
@@ -219,31 +218,13 @@ class ModelsTestCase(RedisTestCaseMixin, ModoTestCase):
         super().setUpTestData()
         admin_factories.populate_database()
 
-    def test_domain_detail_view(self):
-        domain = admin_models.Domain.objects.get(name="test.com")
-        domain.message_limit = 10
-        domain.save()
-        url = reverse("admin:domain_detail", args=[domain.pk])
-        response = self.client.get(url)
-        self.assertContains(response, "Message sending limit")
-
-    def test_account_detail_view(self):
-        account = core_models.User.objects.get(username="user@test.com")
-        mb = account.mailbox
-        mb.message_limit = 10
-        mb.save()
-
-        url = reverse("admin:account_detail", args=[account.pk])
-        response = self.client.get(url)
-        self.assertContains(response, "Message sending limit")
-
     def test_domain_signal_handler(self):
         domain = admin_models.Domain.objects.get(name="test.com")
         domain.message_limit = 10
         domain.save()
         self.assertEqual(
             self.rclient.hget(constants.REDIS_HASHNAME, domain.name),
-            domain.message_limit
+            domain.message_limit,
         )
 
         # Force constructor call to fill _loaded_values
@@ -252,11 +233,9 @@ class ModelsTestCase(RedisTestCaseMixin, ModoTestCase):
         domain.save()
         self.assertEqual(
             self.rclient.hget(constants.REDIS_HASHNAME, domain.name),
-            domain.message_limit
+            domain.message_limit,
         )
 
         domain.message_limit = None
         domain.save()
-        self.assertFalse(
-            self.rclient.hexists(constants.REDIS_HASHNAME, domain.name)
-        )
+        self.assertFalse(self.rclient.hexists(constants.REDIS_HASHNAME, domain.name))
