@@ -1,10 +1,10 @@
 """Transport models."""
 
-import jsonfield
 from reversion import revisions as reversion
 
+from django.core.serializers.json import DjangoJSONEncoder
 from django.db import models
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from . import backends
 
@@ -12,12 +12,11 @@ from . import backends
 class Transport(models.Model):
     """Transport table."""
 
-    pattern = models.CharField(
-        _("pattern"), unique=True, max_length=253, db_index=True)
+    pattern = models.CharField(_("pattern"), unique=True, max_length=253, db_index=True)
     service = models.CharField(_("service"), max_length=30)
     next_hop = models.CharField(_("next hop"), max_length=100, blank=True)
 
-    _settings = jsonfield.JSONField(default={})
+    _settings = models.JSONField(default=dict, encoder=DjangoJSONEncoder)
 
     class Meta:
         ordering = ["pattern"]
